/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.datastore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.rmidriver.EisEvent;
import commonj.sdo.DataObject;

public class DataStore {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}
	private static DataStore model = new DataStore();


	private ArrayList<IDataStoreChangeListener> listenerList = new ArrayList<IDataStoreChangeListener>();

	// For Received List.
	private Vector<RequestDataItem> requestDataItemList = new Vector<RequestDataItem>();

	// For Send List.
	private Vector<EventDataItem> eventDataItemList = new Vector<EventDataItem>();

	// A Map to accelerate the access of DOItem by eventId.
	private HashMap<String, EventDataItem> eventDataItemMap = new HashMap<String, EventDataItem>();
	
	private int replyMode = EisMockerConstaints.REPLY_AUTO;

	private DataStore() {
	}

	// For Received BO List.
	public RequestDataItem addRequestDataItem(DataObject dataObj, String operation) {
		RequestDataItem dataItem = new RequestDataItem(dataObj, operation);
		requestDataItemList.add(dataItem);
		fireReceivedListChanged();
		return dataItem;
	}

	public boolean removeRequestDataItem(RequestDataItem item) {
		boolean flag = requestDataItemList.remove(item);
		fireReceivedListChanged();
		return flag;
	}

	public void clearRequestDataItemList() {
		requestDataItemList.clear();
		fireReceivedListChanged();
	}

	public RequestDataItem[] getRequestDataItems() {
		RequestDataItem[] dataItemArry = new RequestDataItem[requestDataItemList.size()];
		return requestDataItemList.toArray(dataItemArry);
	}
	public int getRequestDataItemCount(){
		return requestDataItemList.size();
	}

	// For Send BO List.
	public ArrayList<EisEvent> getEisEvents() {
		ArrayList<EisEvent> eventList = new ArrayList<EisEvent>();
		Iterator<EventDataItem> iterator = eventDataItemList.iterator();
		while (iterator.hasNext()) {
			EventDataItem item = iterator.next();
			eventList.add(item.getEisEvent());
		}
		return eventList;
	}
	
	public int getEventDataItemCount(){
		return eventDataItemList.size();
	}

	public DataObject getDataObjectByEventId(String eventId) {
		EventDataItem dataItem = (EventDataItem) eventDataItemMap.get(eventId);
		DataObject sdoRt = null;
		if (dataItem != null) {
			sdoRt = dataItem.getDataObject();
		}
		return sdoRt;
	}

	public EisEvent getEisEventByEventId(String eventId) {
		EventDataItem dataItem = (EventDataItem) eventDataItemMap.get(eventId);
		EisEvent eventRt = null;
		if (dataItem != null) {
			eventRt = dataItem.getEisEvent();
		}
		return eventRt;
	}

	public EventDataItem[] getEventDataItems() {
		EventDataItem[] dataArry = new EventDataItem[eventDataItemList.size()];
		return eventDataItemList.toArray(dataArry);
	}

	public boolean addEventDataItem(DataObject dataObj, String operation) {
		EventDataItem dataItem = new EventDataItem(dataObj,operation);
		boolean flag = eventDataItemList.add(dataItem);
		eventDataItemMap.put(dataItem.getEisEvent().getEventId(), dataItem);
		fireSendListChanged();
		return flag;
	}
	public boolean removeEventDataItem(EventDataItem item) {
		boolean flag = false;
		if (item != null) {
			eventDataItemMap.remove(item);
			flag = eventDataItemList.remove(item);
			fireSendListChanged();
		}
		return flag;
	}

	public boolean removeEventDataItemByEventId(String eventId) {
		EventDataItem item = (EventDataItem) eventDataItemMap.get(eventId);
		return removeEventDataItem(item);
	}

	public void clearEventDataItemList() {
		eventDataItemMap.clear();
		eventDataItemList.clear();
		fireSendListChanged();
	}

//	public boolean addBatchSendObject(DataObject dataObj, String operation) {
//		return addSendObject(dataObj, operation);
//	}

	public void updateEventStatus(String eventId, int status) {
		EventDataItem item = (EventDataItem) eventDataItemMap.get(eventId);
		item.getEisEvent().setStatus(status);
		fireSendListChanged();
	}


	public static DataStore getDataStore() {
		return model;
	}

	public void addListener(IDataStoreChangeListener listener) {
		listenerList.add(listener);
	}

	public void removeListener(IDataStoreChangeListener listener) {
		listenerList.remove(listener);
	}

	/**
	 * Update both received list and send List.
	 */
	public void fireModelChanged() {
		for (int i = 0; i < listenerList.size(); i++) {
			IDataStoreChangeListener listener = listenerList.get(i);
			listener.sendListChanged();
			listener.receivedNewItems();
		}
	}

	public void fireReceivedListChanged() {
		for (int i = 0; i < listenerList.size(); i++) {
			IDataStoreChangeListener listener = listenerList.get(i);
			listener.receivedNewItems();
		}
	}

	public void fireSendListChanged() {
		for (int i = 0; i < listenerList.size(); i++) {
			IDataStoreChangeListener listener = listenerList.get(i);
			listener.sendListChanged();
		}
	}
	
	public void setReplyMode(int mode){
		this.replyMode = mode;
	}
	public int getReplyMode(){
		return this.replyMode;
	}
}
