/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.main;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.ibm.eismocker.rmiserver.RMIObjectServer;
import com.ibm.eismocker.ui.EisMockerShell;
import com.ibm.eismocker.utils.EisMockerUtils;
import com.ibm.eismocker.utils.PropertiesUtil;
import com.ibm.eismocker.utils.XSDScanner;

public class EisMockerMain {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}
	private EisMockerShell mainShell;

	private RMIObjectServer rmiServer;

	private Logger logger;

	public static void main(String[] args) {
		if (args.length == 1) {
			EisMockerMain runtime = new EisMockerMain();
			runtime.start(args[0]);
		} else {
			System.out.println("Please specify the argument which indicates the path of the property file");

		}
	}

	private void start(String strFile) {
		try {
			initialize(strFile);
			mainShell.open();
			logger.log(Level.FINER, "Program exit!");
			System.exit(0);
		} catch (Exception e) {
			logger.log(Level.SEVERE, e.getMessage(), e);
		}
	}

	private void initialize(String propertyPath) throws Exception {
		PropertiesUtil properties = new PropertiesUtil(propertyPath, PropertiesUtil.EIS);
		properties.importProperties();
		int serverPort = properties.getPort();
		String workSpace = properties.getWorkSpace();
		String logFile = properties.getLogFile();

		EisMockerUtils.setLogFileName(logFile);
		logger = EisMockerUtils.getLogger();

		String logLevelStr = properties.getLoggerLevel().toUpperCase();
		try {
			Level logLevel = Level.parse(logLevelStr);
			logger.setLevel(logLevel);
		} catch (IllegalArgumentException e) {
			logger.severe("Logger Level error. Use " + logger.getLevel() + " by default.");
		}

		logger.finer("BO Workspace = " + workSpace);

		// Initialize the GUI Shell
		mainShell = new EisMockerShell("EIS Mocker");
		String[] supportedBOList = XSDScanner.initializeSupportedBOList(workSpace);
		mainShell.setSupportedBOList(supportedBOList);
		mainShell.initialize();

		// startup the RMI service
		boolean started = startupRMIService(serverPort);
		if (started) {
			mainShell.setStatus("Start RMI Serivce @ " + serverPort);
		} else {
			mainShell.setStatus("Start RMI Serivce failed!");
			logger.finer("Start RMI Serivce failed.");
		}

	}

	private boolean startupRMIService(int iPort) {
		boolean started = false;
		// start up the rmi service
		String rmiPath = "rmi://localhost:" + iPort + "/DOStoreService";
		try {
			logger.finer("RMI Address: " + rmiPath);
			rmiServer = new RMIObjectServer(rmiPath);
			if (rmiServer != null) {
				rmiServer.getDataObjectStore();
				started = rmiServer.start();
			}
		} catch (Exception e) {
			started = false;
			logger.log(Level.SEVERE, e.getMessage(), e);
		}
		return started;
	}

}
