/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.rmidriver;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.ArrayList;



/** 
 * A class implements <code>Remote</code> implement to provide a RMI-based 
 * objects exchange method between <code>RMIObjectServer</code> and
 * <code>RMIObjectClient</code>.
 *
 */
public interface IDataObjectStore extends Remote{
	public static final String COPYRIGHT=" Copyright IBM Corporation 2008.";  //$NON-NLS-1$
	
	//Methods for inbound operations.
	/**Get a <code>ArrayList</code> stores event metaDatas.
	 * @return	<code>ArrayList</code> stores <code>VAEvent</code>
	 * @throws RemoteException
	 */
	public ArrayList<EisEvent> getEvents() throws RemoteException;
	
	/**Get <code>DataObject</code> in <code>DataMap</code> by given eventId. 
	 * @param eventId	Given id of <code>DataObject</code>
	 * @return <code>DataMap</code>
	 * @throws RemoteException
	 */
	public DataMap getDataMapForEvent(String eventId) throws RemoteException;
		
	/**Update the status code of <code>DOItemMetaData</code>.
	 * @param eventId	Id for <code>DOItemMetaData</code>
	 * @param status	New status code.
	 * @throws RemoteException
	 */
	public void updateEventStatus(String eventId, int status) throws RemoteException;
	
	
	/**Delete the <code>DOItemMetaData</code> with specified eventId.
	 * @param eventId 
	 * @throws RemoteException
	 */
	public void deleteEvent(String eventId) throws RemoteException;
	
	//Methods for outbound operation.
	/** Import a DataObject in DataMap format into the DataObjectStore.  
	 * @param dataMap DataObject in <code>DataMap</code> form.
	 * @param operation an operation <code>String</code>. (Create/Delete/Update)
	 * @return return the same <code>DataMap</code> as input by default.
	 */
	public DataMap sendDataMap(DataMap dataMap, String operation) throws RemoteException;
}
