/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.rmiserver;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.logging.Level;

import com.ibm.eismocker.datastore.DataStore;
import com.ibm.eismocker.datastore.RequestDataItem;
import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.eismocker.rmidriver.IDataObjectStore;
import com.ibm.eismocker.rmidriver.EisEvent;
import com.ibm.eismocker.utils.EisMockerUtils;

import commonj.sdo.DataObject;

public class DataObjectStoreImpl extends UnicastRemoteObject implements IDataObjectStore {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private static final long serialVersionUID = 3579753;

	private DataStore dataStore;

	/**
	 * @param buffer
	 *            The Vector use to save DataObjects.
	 * @throws RemoteException
	 */
	public DataObjectStoreImpl() throws RemoteException {
		super();
		dataStore = DataStore.getDataStore();
	}

	// Inbound Method.
	public void deleteEvent(String eventId) throws RemoteException {
		EisMockerUtils.getLogger().entering("DataObjectStoreImpl", "deleteEvent");
		try {
			dataStore.removeEventDataItemByEventId(eventId);
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			throw new RemoteException(e.getMessage(), e);
		}
		EisMockerUtils.getLogger().exiting("DataObjectStoreImpl", "deleteEvent");
	}

	public ArrayList<EisEvent> getEvents() throws RemoteException {
		EisMockerUtils.getLogger().entering("DataObjectStoreImpl", "getEvents");
		ArrayList<EisEvent> eventList = null;
		try {
			eventList = dataStore.getEisEvents();
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			throw new RemoteException(e.getMessage(), e);
		}
		EisMockerUtils.getLogger().exiting("DataObjectStoreImpl", "getEvents");
		return eventList;
	}

	public DataMap getDataMapForEvent(String eventId) throws RemoteException {
		EisMockerUtils.getLogger().entering("DataObjectStoreImpl", "getDataMapForEvent");
		DataMap dataMap = null;
		try {
			DataObject dataObj = dataStore.getDataObjectByEventId(eventId);
			dataMap = EisMockerUtils.sdo2DataMap(dataObj);
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			throw new RemoteException(e.getMessage(), e);
		}
		EisMockerUtils.getLogger().exiting("DataObjectStoreImpl", "getDataMapForEvent");
		return dataMap;
	}

	public void updateEventStatus(String eventId, int status) throws RemoteException {
		EisMockerUtils.getLogger().entering("DataObjectStoreImpl", "updateEventStatus");
		try {
			dataStore.updateEventStatus(eventId, status);
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			throw new RemoteException(e.getMessage(), e);
		}
		EisMockerUtils.getLogger().exiting("DataObjectStoreImpl", "updateEventStatus");
	}

	// Outbound Method.
	public DataMap sendDataMap(DataMap dataMap, String operation) throws RemoteException {
		EisMockerUtils.getLogger().entering("DataObjectStoreImpl", "sendDataMap");
		DataMap responseDataMap = null;
		try {
			DataObject dataObj = EisMockerUtils.dataMap2Sdo(dataMap);
			RequestDataItem dataItem = dataStore.addRequestDataItem(dataObj, operation);
			if (dataStore.getReplyMode() == EisMockerConstaints.REPLY_MANU) { // manually-reply
				synchronized (dataItem) {
					dataItem.wait();
					DataObject responseSDO = dataItem.getResponseSDO();
					responseDataMap = EisMockerUtils.sdo2DataMap(responseSDO);
				}
			} else {// auto-reply
				dataItem.setIsReplied(true);
				//responseDataMap = dataMap;
				DataObject responseSDO = dataItem.getResponseSDO();
				responseDataMap = EisMockerUtils.sdo2DataMap(responseSDO);	
				dataStore.fireReceivedListChanged();
			}
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			throw new RemoteException(e.getMessage(), e);
		}
		EisMockerUtils.getLogger().exiting("DataObjectStoreImpl", "sendDataMap");
		return responseDataMap;
	}

}
