/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.sdoviewer;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.List;
import java.util.logging.Level;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TreeItem;

import com.ibm.eismocker.ui.EisMockerShell;
import com.ibm.eismocker.utils.EisMockerUtils;
import commonj.sdo.DataObject;
import commonj.sdo.Property;

public class SDOTreeCellModifier implements ICellModifier {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private TreeViewer parentTreeViewer = null;

	public SDOTreeCellModifier(TreeViewer treeViewer) {
		parentTreeViewer = treeViewer;
	}

	public boolean canModify(Object element, String property) {
		if (SDOTreeConstants.TREE_VALUE.equals(property)) {
			SDOTreeItem treeItem = (SDOTreeItem) element;
			if (treeItem.isContainment() || treeItem.isDataObjectNode()) {
				return false;
			}
			return true;
			//return false;
		}
		return false;
	}

	public Object getValue(Object element, String property) {
		Object rtValue = null;
		if (SDOTreeConstants.TREE_VALUE.equals(property)) {
			Object obj = ((SDOTreeItem) element).getObject();
			if (obj == null) {
				rtValue = "";
			} else {
				rtValue = obj.toString();
			}
		}
		return rtValue;
	}

	@SuppressWarnings("unchecked")
	public void modify(Object element, String property, Object value) {
		if ((value != null) && SDOTreeConstants.TREE_VALUE.equals(property)) {
			// This element must be a children object.
			TreeItem item = (TreeItem) element;
			SDOTreeItem treeItem = (SDOTreeItem) item.getData();
			SDOTreeItem parentItem = (SDOTreeItem) treeItem.getParent();
			Property prop = treeItem.getProperty();
			Property parentProp = parentItem.getProperty();

			try {
				if ((parentProp != null) && parentProp.isMany() && (!parentProp.isContainment())) {
					// parent is many
					List<Object> list = (List<Object>) parentItem.getObject();
					int index = list.indexOf(treeItem.getObject());
					if (index >= 0) {
						Object newValue = convertPropertyValue(value, prop);
						list.set(index, newValue);
						treeItem.setObject(newValue);
					}
				} else { // parent is null or is not many
					DataObject dataObj = (DataObject) parentItem.getObject();
					String strValue = (String) value;
					if (!dataObj.isSet(prop) && strValue.equals("")) {
						return;
					} else {
						Object newValue = convertPropertyValue(value, prop);
						dataObj.set(prop, newValue);
						treeItem.setObject(newValue);
					}
				}
				parentTreeViewer.refresh(treeItem, true);
			} catch (NumberFormatException nfe) {
				MessageBox messageBox = new MessageBox(EisMockerShell.getShell(), SWT.ICON_WARNING);
				messageBox.setText("Warning!");
				messageBox.setMessage("The number format is invalid: "+nfe.getMessage());
				messageBox.open();
				EisMockerUtils.getLogger().log(Level.FINE, nfe.getMessage(), nfe);
			} catch (Exception e) {
				EisMockerUtils.getLogger().log(Level.FINE, e.getMessage(), e);
			}
		}
	}

	private Object convertPropertyValue(Object value, Property property) throws Exception {

		Object newValue = null;
		if ((value == null) || (value.toString().trim().equals(""))) {
			return newValue;
		}
		String strVal = value.toString();
		String typeName = property.getType().getName();
		if (typeName.startsWith("Boolean")) {// For Boolean and BooleanObject
			newValue = new Boolean(strVal);
		} else if (typeName.equals("Integer")) {
			newValue = new BigInteger(strVal);
		} else if (typeName.startsWith("Int")) {// For Int and IntObject
			newValue = new Integer(strVal);
		} else if (typeName.equals("Byte")) {
			newValue = new Byte(strVal);
		} else if (typeName.startsWith("Float")) {// for Float and FloatObject
			newValue = new Float(strVal);
		} else if (typeName.equals("Long")) {
			newValue = new Long(strVal);
		} else if (typeName.equals("Short")) {
			newValue = new Short(strVal);
		} else if (typeName.startsWith("Double")) {// for Double and DoubleObject
			newValue = new Double(strVal);
		} else if (typeName.equals("Decimal")) {
			newValue = new BigDecimal(strVal);
		} else if (typeName.equals("Bytes")) {
			// TODO: Problems occurred here. As we can not use String for this type.
			newValue = strVal.getBytes();
		} else if (typeName.equals("String")) {
			// remove invalid character
			newValue = value;
		} else if( typeName.endsWith("verb")){ // is work-as-design or defect for SDO2 ?
			//property.getType().getName() returns "verb", not "String"
			newValue = value;
		} else{
			newValue = value;
		}

		return newValue;
	}

}
