/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.sdoviewer;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.ibm.eismocker.utils.EisMockerUtils;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

public class SDOTreeContentProvider implements ITreeContentProvider {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	@SuppressWarnings("unchecked")
	public Object[] getChildren(Object obj) {
		SDOTreeItem treeItem = (SDOTreeItem) obj;
		ArrayList<SDOTreeItem> childrenList = new ArrayList<SDOTreeItem>();

		// For SDOTreeContainerItem.
		if (treeItem.isContainment()) {
			return ((SDOTreeContainerItem) treeItem).getChildren();
		}
		if (!treeItem.isDataObjectNode()) {
			return null;
		}
		DataObject dataObject = (DataObject) treeItem.getObject();

		Type type = dataObject.getType();
		List<Property> list = type.getProperties();

		for (int i = 0; i < list.size(); i++) {
			Property property = list.get(i);
			// For many-valued properties, process a list of values.
			if (property.isMany()) {
				List<?> values = dataObject.getList(i);
				SDOTreeContainerItem containerItem = new SDOTreeContainerItem(values, property, treeItem);

				// Add all children items to the containerItem.
				ArrayList<SDOTreeItem> tempChildList = new ArrayList<SDOTreeItem>();
				for (int v = 0, count = values.size(); v < count; v++) {
					SDOTreeItem childItem = new SDOTreeItem(values.get(v), property, treeItem);
					tempChildList.add(childItem);
				}
				containerItem.addAll(tempChildList);
				childrenList.add(containerItem);
				continue;
			} else {
				if (!(getDataObjectTypeName(property.getType()).toUpperCase().equals("ANYTYPE")) && !treeItem.isUnset(property.getName())) {
					if (property.isContainment()) {
						// By default, create a new instance for Every
						// DataObject property.
						Object childObj = dataObject.get(property);
						if (childObj == null) {
							dataObject.createDataObject(property);
						}
					} else {
						Object elementObj = dataObject.get(property);
						if (elementObj == null) {
							dataObject.set(i, initializeSimpleTypeValue(property));
						}
					}
				}
			}
			// Add a new Child item.
			childrenList.add(new SDOTreeItem(dataObject.get(i), property, treeItem));
		}
		return childrenList.toArray();
	}

	private Object initializeSimpleTypeValue(Property property) {
		Object obj = null;

		return obj;
	}

	public Object getParent(Object dOTreeItem) {
		return ((SDOTreeItem) dOTreeItem).getParent();
	}

	@SuppressWarnings("unchecked")
	public boolean hasChildren(Object obj) {
		SDOTreeItem treeItem = (SDOTreeItem) obj;

		if (treeItem.isContainment()) {
			SDOTreeContainerItem item = (SDOTreeContainerItem) treeItem;
			return (item.getChildrenCounts() > 0);
		}
		if (!treeItem.isDataObjectNode()) {
			return false;
		}
		DataObject dataObject = (DataObject) treeItem.getObject();
		Type type = dataObject.getType();
		List<Property> list = type.getProperties();

		if (list.size() > 0)
			return true;

		return false;
	}

	public Object[] getElements(Object obj) {
		return new Object[] { ((SDOTreeItem) obj).getObject() };
	}

	public void dispose() {
	}

	public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
	}

	private String getDataObjectTypeName(Type type) {
		String typeName = EisMockerUtils.getBaseTypeName(type);
		if (typeName == null) {
			return "";
		}

		if (SDOTreeXSDMapper.containsKey(typeName)) {
			typeName = SDOTreeXSDMapper.getType(typeName);
		}

		return typeName;
	}
	

}
