/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.sdoviewer;

import java.util.ArrayList;

import commonj.sdo.DataObject;
import commonj.sdo.Property;

public class SDOTreeItem {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private Object object;
	private Property property;
	private SDOTreeItem parentItem;
	private ArrayList<String> unsetList;
	
	public static final SDOTreeItem ROOT_ITEM = new SDOTreeItem(null,null,null);
	
	public SDOTreeItem(Object obj, Property property, SDOTreeItem parentItem){
		this.object = obj;
		this.property = property;
		this.parentItem = parentItem;	
		unsetList = new ArrayList<String>();
		unsetList.clear();
	}
	
	public void unset(String name){
		unsetList.add(name);
	}
	
	public boolean isUnset(String name){
		return unsetList.contains(name);
	}
	
	public Object getObject() {
		return object;
	}
	
	public void setObject(Object obj){
		this.object = obj;
		unsetList.clear();
	}
	
	public SDOTreeItem getParent() {
		return parentItem;
	}
	
	public void setParent(SDOTreeItem item){
		this.parentItem = item;
	}
	
	public Property getProperty(){
		return this.property;
	}
	
	public boolean isDataObjectNode(){
		if(object instanceof DataObject){
			return true;
		}
		return false;
	}
	
	/**
	 * Type method uses to indicate the TreeItem is containter or not.
	 * Container is a special tree node just for presentation.
	 * 	e.g. Wbiphone[2] ----- Wibphone[0]
	 *
	 *				   	---- Wibphone[1]
	 *	In this example, Node Wibphone[2] is a container node and Wibphone[0] 
	 *	and Wibphone[1] are normal node.
	 *
	 *	Note: Different return value will only implements in subclass.
	 * @return 
	 */
	public boolean isContainment(){
		return false;
	}
}
