/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.sdoviewer;

import java.util.Calendar;
import java.util.Date;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import com.ibm.eismocker.utils.EisMockerUtils;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

public class SDOTreeLabelProvider extends LabelProvider implements ITableLabelProvider {
	static String copyright() {
		return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	public String getColumnText(Object obj, int lineNum) {
		SDOTreeItem item = (SDOTreeItem) obj;
		Property property = item.getProperty();
		String strRt = "";

		switch (lineNum) {
		case 0:// Attribute Name column
			if (property == null) { // Root object - Data Graph
				strRt = ((DataObject) item.getObject()).getType().getName();
			} else if (item.getParent().isContainment()) {
				SDOTreeContainerItem containerItem = (SDOTreeContainerItem) item.getParent();
				strRt = property.getName() + "[" + containerItem.getIndex(item) + "]";
			} else if (item.isContainment()) {
				strRt = property.getName() + "[" + ((SDOTreeContainerItem) item).getChildrenCounts() + "]";
			} else {// Simple type
				strRt = property.getName();
			}
			break;
			
		case 1:// Attribute Type column
			Type type;
			// add second part judgement to enhance the real-time DO type update
			if (property == null || (item.getObject() != null && item.getObject() instanceof DataObject)) {
				type = ((DataObject) item.getObject()).getType();
			} else {
				type = property.getType();
			}
			// Get base Type if base Type availiable.
			String typeName = EisMockerUtils.getBaseTypeName(type);
			if (SDOTreeXSDMapper.containsKey(typeName)) {
				typeName = SDOTreeXSDMapper.getType(typeName);
			}
			if (item.isContainment()) {
				typeName = typeName + "[]";
			}
			strRt = typeName;
			break;

		case 2:// Attribute Value column
			if (item.isContainment() || item.isDataObjectNode()) {
				strRt ="";
			} else {
				Object objValue = item.getObject();
				if (objValue != null) {					
					Object propType = objValue.getClass();
					if(propType == Date.class || propType == Calendar.class){
						//convert the value according to the data type
						//reserved for future using.
						strRt = objValue.toString();
					}else{
						strRt = objValue.toString();
					}
				}
			}
			break;

		default:

		}
		return strRt;
	}

	public Image getColumnImage(Object arg0, int arg1) {
		return null;
	}
}
