/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.sdoviewer;

import java.util.List;
import java.util.logging.Level;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import com.ibm.eismocker.utils.EisMockerUtils;

import commonj.sdo.DataObject;
import commonj.sdo.Property;

public class SDOTreeSelectionChangedListener implements ISelectionChangedListener {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private TreeViewer parentTreeViewer = null;

	public SDOTreeSelectionChangedListener(TreeViewer treeViewer) {
		this.parentTreeViewer = treeViewer;
	}

	public void selectionChanged(SelectionChangedEvent evt) {
		ISelection selection = evt.getSelection();
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			SDOTreeItem curTreeItem = (SDOTreeItem) structuredSelection.getFirstElement();

			if (curTreeItem != null) {
				if (curTreeItem.isContainment()) {
					parentTreeViewer.getTree().setMenu(createTreeItemAddMenu(curTreeItem));
				} else {
					if (curTreeItem.getParent() instanceof SDOTreeContainerItem) {
						parentTreeViewer.getTree().setMenu(createTreeItemDeleteMenu(curTreeItem));
					} else {
						parentTreeViewer.getTree().setMenu(null);
					}
				}
			}
		}
	}

	private Menu createTreeItemAddMenu(SDOTreeItem curTreeItem) {
		// Shell shell = this.parentTreeViewer.getTree().getParent();
		final SDOTreeItem treeItem = curTreeItem;
		Menu menu = new Menu((Shell)this.parentTreeViewer.getTree().getParent().getShell(), SWT.POP_UP | SWT.NO_RADIO_GROUP);
		MenuItem addItem = new MenuItem(menu, SWT.PUSH);
		addItem.setText("Add");
		addItem.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent evt) {
				try {
					if (treeItem != null) {
						// Create a new child DataObject if property is a
						// containment type.
						Property property = treeItem.getProperty();
						if (property != null) {
							DataObject parentDO = (DataObject) treeItem.getParent().getObject();
							SDOTreeContainerItem item = (SDOTreeContainerItem) treeItem;
							if (property.isContainment()) {
								//DataObject childDO = DataObjectUtil.createChildBO(parentDO, property);
								DataObject childDO = parentDO.createDataObject(property);
								item.add(new SDOTreeItem(childDO, property, treeItem));
							} else {
								List<?> pList = parentDO.getList(property);
								pList.add(null);
								item.add(new SDOTreeItem(null, property, treeItem));
							}
							parentTreeViewer.refresh(treeItem, true);
						}
					}
				} catch (Exception e) {
					EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
				}
			}

			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});
		return menu;
	}

	private Menu createTreeItemDeleteMenu(SDOTreeItem curTreeItem) {
		final SDOTreeItem treeItem = curTreeItem;
		Menu menu = new Menu((Shell) this.parentTreeViewer.getTree().getParent().getShell(), SWT.POP_UP | SWT.NO_RADIO_GROUP);
		MenuItem deleteItem = new MenuItem(menu, SWT.PUSH);
		deleteItem.setText("Delete");
		deleteItem.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent evt) {
			}

			public void widgetSelected(SelectionEvent evt) {
				if (treeItem != null) { // Delete the selected TreeItem.
					SDOTreeContainerItem parentTreeItem = (SDOTreeContainerItem) treeItem.getParent();
					parentTreeItem.remove(treeItem);
					DataObject curSDO = (DataObject) treeItem.getObject();
					if (curSDO != null) {
						curSDO.delete();
					}
					parentTreeViewer.refresh(parentTreeItem, true);
					parentTreeViewer.expandToLevel(parentTreeItem, 1);
				}
			}
		});
		return menu;
	}
}
