/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.sdoviewer;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TreeColumn;

import commonj.sdo.DataObject;

public class SDOTreeViewer extends TreeViewer {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private DataObject sdoContent = null;

	public SDOTreeViewer(Composite comp) {
		super(comp, SWT.FULL_SELECTION | SWT.BORDER);
		initialize();
	}

	
	public boolean setContentSDO(DataObject sdo) {
		sdoContent = sdo;
		if (sdoContent != null) {
			// Create a new root TreeItem.
			SDOTreeItem rootTreeItem = buildTreeItem(sdoContent);
			this.setInput(rootTreeItem);
			this.refresh(true);
			this.expandToLevel(3);
			return true;
		} else {
			this.setInput(null);
			this.refresh(true);
			return false;
		}
	}
	public DataObject getContentSDO(){
		return sdoContent;
	}

	private SDOTreeItem buildTreeItem(DataObject dataObject) {
		SDOTreeItem node = new SDOTreeItem(dataObject, null, SDOTreeItem.ROOT_ITEM);
		return new SDOTreeItem(node, null, null);
	}

	private void initialize() {
		GridData treeViewerData = new GridData(GridData.FILL_BOTH);
		this.getTree().setLayoutData(treeViewerData);

		this.getTree().setHeaderVisible(true);
		this.getTree().setLinesVisible(true);

		// Build Columns.
		createColumn(SDOTreeConstants.TREE_NAME, 176, SWT.CENTER);
		createColumn(SDOTreeConstants.TREE_TYPE, 100, SWT.CENTER);
		createColumn(SDOTreeConstants.TREE_VALUE, 120, SWT.BEGINNING);

		// Set the layout data of ListViewer.
		this.setContentProvider(new SDOTreeContentProvider());
		this.setLabelProvider(new SDOTreeLabelProvider());

		// Initialize the CellEditor.
		this.setCellModifier(new SDOTreeCellModifier(this));
		CellEditor[] editors = new CellEditor[3];
		//change it to use ComboxCellEditor later
		editors[2] = new TextCellEditor(this.getTree());
		//editors[2] = new ComboBoxCellEditor(this.getTree(), new String[]{"bcd", "unset","abc" },SWT.READ_ONLY);
		//done
		
		this.setCellEditors(editors);
		this.setColumnProperties(new String[] { SDOTreeConstants.TREE_NAME, SDOTreeConstants.TREE_TYPE, SDOTreeConstants.TREE_VALUE });

		// Set listener.
		this.addSelectionChangedListener(new SDOTreeSelectionChangedListener(this));
	}

	private TreeColumn createColumn(String label, int width, int style) {
		TreeColumn column = new TreeColumn(this.getTree(), style);
		column.setText(label);
		column.setWidth(width);
		
		column.setMoveable(false);
		column.setToolTipText(label);
		
		return column;
	}

}
