/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.ui;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.logging.Level;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Sash;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import com.ibm.eismocker.datastore.DataStore;
import com.ibm.eismocker.datastore.EventDataItem;
import com.ibm.eismocker.datastore.IDataStoreChangeListener;
import com.ibm.eismocker.datastore.RequestDataItem;
import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.sdoviewer.SDOTreeViewer;
import com.ibm.eismocker.ui.batch.BatchSendBOShell;
import com.ibm.eismocker.ui.responsebo.ResponseBOShell;
import com.ibm.eismocker.utils.EisMockerUtils;
import com.ibm.eismocker.utils.ImageManager;
import com.ibm.eismocker.utils.XSDScanner;
import commonj.sdo.DataObject;
import commonj.sdo.helper.CopyHelper;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.XMLHelper;

public class EisMockerShell implements IDataStoreChangeListener {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private static Shell shell = null;

	private Sash sash = null;

	private Control fileToolBar = null;

	private static Display display = null;

	private String[] supported_bo_list = null;

	private SDOTreeViewer sdoTreeViewer = null;

	private Combo boTypeCombo = null;

	private Combo opTypeCombo = null;

	private Label statusLabel = null;

	private ListViewer sendListViewer = null;

	private ListViewer receivedListViewer = null;

	private CTabFolder cFolder = null;

	private DataStore dataStore = null;

	// private Logger logUtil = null;

	private RequestDataItem curReceivedListSelectedItem = null;

	private EventDataItem curSentListSelectedItem = null;

	public EisMockerShell(String title) {
		dataStore = DataStore.getDataStore();

		display = new Display();
		shell = new Shell(display);
		shell.setSize(600, 450);
		shell.setText(title);
		shell.setLayout(new GridLayout());
		shell.setImage(ImageManager.get("MainIcon"));
	
		// add listener for the data store
		dataStore.addListener(this);
	}

	public void initialize() {
		EisMockerUtils.getLogger().entering("EisMockerShell", "initialize");

		createMenu();
		createToolBar();

		// Create Content Composite
		Composite contentComposite = new Composite(shell, SWT.NONE);
		contentComposite.setLayout(new FormLayout());
		GridData contentData = new GridData(GridData.FILL_BOTH | GridData.VERTICAL_ALIGN_FILL);
		contentComposite.setLayoutData(contentData);

		// Create recieved/send BO list
		createBOList(contentComposite);

		createBOEditor(contentComposite);

		// Status Label Composite
		Composite statusLabelComposite = new Composite(shell, SWT.NONE);
		GridData statusData = new GridData(GridData.VERTICAL_ALIGN_END | GridData.HORIZONTAL_ALIGN_FILL);
		statusLabelComposite.setLayoutData(statusData);
		statusLabelComposite.setLayout(new FillLayout());
		statusLabel = new Label(statusLabelComposite, SWT.NONE);
		statusLabel.setText("hello");

		initReceivedListViewer(dataStore);
		initSentListViewer(dataStore);
		initSash(contentComposite);
		
		EisMockerUtils.getLogger().exiting("EisMockerShell", "initialize");
	}

	private void createBOEditor(Composite contentComposite) {
		EisMockerUtils.getLogger().entering("EisMockerShell", "createBOEditor");
		
		Composite boEditorComposite = new Composite(contentComposite, SWT.NONE);
		FormData treeCompositeData = new FormData();
		treeCompositeData.left = new FormAttachment(sash, 0);
		treeCompositeData.right = new FormAttachment(100, 0);
		treeCompositeData.top = new FormAttachment(fileToolBar, 5);
		treeCompositeData.bottom = new FormAttachment(99, 0);
		boEditorComposite.setLayoutData(treeCompositeData);
		boEditorComposite.setLayout(new GridLayout());

		Composite labelComposite = new Composite(boEditorComposite, SWT.NONE);
		GridData labelData = new GridData();
		labelData.verticalAlignment = GridData.VERTICAL_ALIGN_BEGINNING;
		labelData.heightHint = 50;
		labelComposite.setLayoutData(labelData);
		GridLayout labelGridLayout = new GridLayout();
		labelGridLayout.numColumns = 4;
		labelGridLayout.makeColumnsEqualWidth = true;
		labelComposite.setLayout(labelGridLayout);

		// Supported BO list
		GridData subData = new GridData(GridData.FILL_BOTH);
		subData = new GridData(GridData.FILL_BOTH);
		subData.grabExcessVerticalSpace = true;
		subData.horizontalSpan = 1;
		subData.heightHint = 23;
		subData.verticalAlignment = GridData.VERTICAL_ALIGN_END;
		subData.horizontalAlignment = GridData.HORIZONTAL_ALIGN_BEGINNING;
		Label boLabel = new Label(labelComposite, SWT.CENTER);
		boLabel.setText(EisMockerConstaints.BO_TYPE);
		boLabel.setLayoutData(subData);

		boTypeCombo = new Combo(labelComposite, SWT.CENTER | SWT.READ_ONLY);
		if (supported_bo_list != null) {
			boTypeCombo.setItems(supported_bo_list);
		} else {
			boTypeCombo.setItems(new String[] { "" });
		}
		boTypeCombo.select(0);
		subData = new GridData(GridData.FILL_BOTH);
		subData.horizontalSpan = 2;
		boTypeCombo.setLayoutData(subData);

		// Create button
		Button createButton = new Button(labelComposite, SWT.FLAT);
		createButton.setText("Create");
		createButton.setToolTipText("Create Business Object");
		createButton.addSelectionListener(new BOActionListener(EisMockerConstaints.ACTION_NEW, this));

		subData = new GridData(GridData.FILL_BOTH);
		subData.grabExcessVerticalSpace = true;
		subData.heightHint = 23;
		subData.horizontalSpan = 1;
		subData.horizontalAlignment = GridData.HORIZONTAL_ALIGN_CENTER;
		createButton.setLayoutData(subData);

		// Supported Operation list
		Label operationLabel = new Label(labelComposite, SWT.CENTER);
		operationLabel.setText(EisMockerConstaints.OP_TYPE);
		subData = new GridData(GridData.FILL_BOTH);
		subData.grabExcessVerticalSpace = true;
		subData.horizontalSpan = 1;
		subData.heightHint = 23;
		subData.verticalAlignment = GridData.VERTICAL_ALIGN_END;
		subData.horizontalAlignment = GridData.HORIZONTAL_ALIGN_BEGINNING;
		operationLabel.setLayoutData(subData);

		opTypeCombo = new Combo(labelComposite, SWT.CENTER | SWT.READ_ONLY);
		opTypeCombo.setItems(EisMockerConstaints.SUPPORTED_OPS);
		opTypeCombo.select(0);
		subData = new GridData(GridData.FILL_BOTH);
		subData.horizontalSpan = 2;
		opTypeCombo.setLayoutData(subData);

		// Create sdo tree viewer
		sdoTreeViewer = new SDOTreeViewer(boEditorComposite);

		EisMockerUtils.getLogger().exiting("EisMockerShell", "createBOEditor");
	}

	public static Display getDisplay() {
		return display;
	}
	public static Shell getShell() {
		return shell;
	}

	public void open() {
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	private void createToolBar() {
		EisMockerUtils.getLogger().entering("EisMockerShell", "createToolBar");
		// Tool Bar Composite
		Composite toolBarComposite = new Composite(shell, SWT.NONE);
		GridLayout toolBarLayout = new GridLayout();
		toolBarLayout.marginHeight = 0;
		toolBarComposite.setLayout(toolBarLayout);

		ToolBar toolBar = new ToolBar(toolBarComposite, SWT.FLAT);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.widthHint = SWT.DEFAULT;
		toolBar.setLayoutData(data);

		addToolItem(toolBar, "New", "New", new BOActionListener(EisMockerConstaints.ACTION_NEW, this));
		addToolItem(toolBar, "Load", "Load", new BOActionListener(EisMockerConstaints.ACTION_LOAD, this));
		addToolItem(toolBar, "Save", "Save", new BOActionListener(EisMockerConstaints.ACTION_SAVE, this));
		new ToolItem(toolBar, SWT.SEPARATOR);
		addToolItem(toolBar, "Remove", "Remove", new BOActionListener(EisMockerConstaints.ACTION_REMOVE, this));
		addToolItem(toolBar, "Clear", "Clear", new BOActionListener(EisMockerConstaints.ACTION_CLEAR, this));
		new ToolItem(toolBar, SWT.SEPARATOR);
		addToolItem(toolBar, "Send", "Send", new BOActionListener(EisMockerConstaints.ACTION_SEND, this));
		addToolItem(toolBar, "Batch Send", "BatchSend", new BOActionListener(EisMockerConstaints.ACTION_BATCH_SEND, this));

		fileToolBar = toolBar;
		EisMockerUtils.getLogger().exiting("EisMockerShell", "createToolBar");
	}

	private void addToolItem(ToolBar toolBar, String toolTipText, String imageKey, SelectionListener listener) {
		ToolItem item = new ToolItem(toolBar, SWT.PUSH);
		item.setToolTipText(toolTipText);
		item.setImage(ImageManager.get(imageKey));
		item.addSelectionListener(listener);
	}

	private void createMenu() {
		EisMockerUtils.getLogger().entering("EisMockerShell", "createMenu");

		Menu menuBar = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menuBar);

		// Initialize File Menu.
		Menu fileSubMenu = addSubMenu(shell, menuBar, EisMockerConstaints.MENU_FILE);
		addMenuItem(fileSubMenu, EisMockerConstaints.MENU_FILE_LOAD, new BOActionListener(EisMockerConstaints.ACTION_LOAD, this));
		addMenuItem(fileSubMenu, EisMockerConstaints.MENU_FILE_SAVE, new BOActionListener(EisMockerConstaints.ACTION_SAVE, this));
		new MenuItem(fileSubMenu, SWT.SEPARATOR);
		addMenuItem(fileSubMenu, EisMockerConstaints.MENU_FILE_EXIT, new BOActionListener(EisMockerConstaints.ACTION_EXIT, this));

		// Initialize Edit Menu.
		Menu editSubMenu = addSubMenu(shell, menuBar, EisMockerConstaints.MENU_EDIT);
		addMenuItem(editSubMenu, EisMockerConstaints.MENU_EDIT_CREATE, new BOActionListener(EisMockerConstaints.ACTION_NEW, this));
		addMenuItem(editSubMenu, EisMockerConstaints.MENU_EDIT_DELETE, new BOActionListener(EisMockerConstaints.ACTION_REMOVE, this));
		addMenuItem(editSubMenu, EisMockerConstaints.MENU_EDIT_CLEAR, new BOActionListener(EisMockerConstaints.ACTION_CLEAR, this));

		// Initialize Request Menu.
		Menu requestSubMenu = addSubMenu(shell, menuBar, EisMockerConstaints.MENU_REQUEST);
		addMenuItem(requestSubMenu, EisMockerConstaints.MENU_REQUEST_SEND, new BOActionListener(EisMockerConstaints.ACTION_SEND, this));
		addMenuItem(requestSubMenu, EisMockerConstaints.MENU_REQUEST_SEND_BATCH, new BOActionListener(EisMockerConstaints.ACTION_BATCH_SEND, this));
		new MenuItem(requestSubMenu, SWT.SEPARATOR);		
		addMenuItem(requestSubMenu, EisMockerConstaints.MENU_REQUEST_REPLY_SUCCESS, new BOActionListener(EisMockerConstaints.ACTION_REPLY, this));
		addMenuItem(requestSubMenu, EisMockerConstaints.MENU_REQUEST_REPLY_FAIL, new BOActionListener(EisMockerConstaints.ACTION_REPLY, this));
		Menu modeSubMenu = addSubMenu(shell, requestSubMenu, EisMockerConstaints.MENU_REQUEST_REPLY_MODE);
		MenuItem item = new MenuItem(modeSubMenu, SWT.RADIO);
		item.setText(EisMockerConstaints.MENU_MODE_AUTOMATIC);
		item.setSelection(true);
		item.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent evt) {
				boolean blCheck = ((MenuItem) evt.getSource()).getSelection();
				if (blCheck) {
					dataStore.setReplyMode(EisMockerConstaints.REPLY_AUTO);
				} else {
					dataStore.setReplyMode(EisMockerConstaints.REPLY_MANU);
				}
			}
		});

		item = new MenuItem(modeSubMenu, SWT.RADIO);
		item.setText(EisMockerConstaints.MENU_MODE_MANUAL);

		// Initialize Help Menu.
		Menu aboutSubMenu = addSubMenu(shell, menuBar, EisMockerConstaints.MENU_HELP);
		addMenuItem(aboutSubMenu, EisMockerConstaints.MENU_HELP_ABOUT, new BOActionListener(EisMockerConstaints.ACTION_ABOUT, this));

		EisMockerUtils.getLogger().exiting("EisMockerShell", "createMenu");
	}

	private Menu addSubMenu(Shell shell, Menu pMenu, String text) {
		MenuItem item = new MenuItem(pMenu, SWT.CASCADE);
		item.setText(text);
		Menu subMenu = new Menu(shell, SWT.DROP_DOWN);
		item.setMenu(subMenu);
		return subMenu;
	}

	private void addMenuItem(Menu menu, String text, SelectionListener listener) {
		MenuItem item = new MenuItem(menu, SWT.PUSH);
		item.setText(text);
		item.addSelectionListener(listener);
	}

	private void createBOList(Composite content) {
		EisMockerUtils.getLogger().entering("EisMockerShell", "createBOList");

		// Create the TabFolder
		cFolder = new CTabFolder(content, SWT.BORDER);
		// Set TabFolder Color and display style.
		cFolder.setSimple(false);
		int colorCount = 3;
		Color[] colors = new Color[colorCount];
		colors[0] = display.getSystemColor(SWT.COLOR_TITLE_BACKGROUND);
		colors[1] = display.getSystemColor(SWT.COLOR_TITLE_BACKGROUND_GRADIENT);
		colors[2] = colors[0];
		int[] percents = new int[colorCount - 1];
		percents[0] = 4;
		percents[1] = 60;
		cFolder.setSelectionBackground(colors, percents, true);
		cFolder.setSelectionForeground(display.getSystemColor(SWT.COLOR_TITLE_FOREGROUND));

		// Create TabItem for Received BO List
		CTabItem receiveBOTab = new CTabItem(cFolder, SWT.NONE);
		receiveBOTab.setText(EisMockerConstaints.RECEIVE_TAB);
		receivedListViewer = new ListViewer(cFolder, SWT.FULL_SELECTION | SWT.BORDER | SWT.V_SCROLL);
		receiveBOTab.setControl(receivedListViewer.getControl());
		cFolder.setFocus();
		receivedListViewer.addDoubleClickListener(new BOListViewerDoubleClickListener(this));
		receiveBOTab.setToolTipText(EisMockerConstaints.RECEIVE_TAB_TIPS);

		// Create TabItem for Send BO List.
		CTabItem sendBOTab = new CTabItem(cFolder, SWT.NONE);
		sendBOTab.setText(EisMockerConstaints.SEND_TAB);
		sendListViewer = new ListViewer(cFolder, SWT.FULL_SELECTION | SWT.BORDER | SWT.V_SCROLL);
		sendBOTab.setControl(sendListViewer.getControl());
		sendBOTab.setToolTipText(EisMockerConstaints.SEND_TAB_TIPS);
		
		// Set the layout
		sash = new Sash(content, SWT.VERTICAL);
		FormData tabLayoutData = new FormData();
		tabLayoutData.right = new FormAttachment(sash, 0);
		tabLayoutData.width = 200;
		cFolder.setLayoutData(tabLayoutData);

		FormData folderData = new FormData();
		folderData.left = new FormAttachment(1, 0);
		folderData.right = new FormAttachment(sash, 0);
		folderData.top = new FormAttachment(0, 5);
		folderData.bottom = new FormAttachment(99, 0);
		cFolder.setLayoutData(folderData);

		EisMockerUtils.getLogger().exiting("EisMockerShell", "createBOList");
	}

	private void initSash(final Composite contentComposite) {
		EisMockerUtils.getLogger().entering("EisMockerShell", "initSash");

		final int limit = 20, percent = 25;
		final FormData sashData = new FormData();
		sashData.left = new FormAttachment(percent, 0);
		sashData.top = new FormAttachment(fileToolBar, 0);
		sashData.bottom = new FormAttachment(99, 0);
		sash.setLayoutData(sashData);
		sash.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				Rectangle sashRect = sash.getBounds();
				Rectangle shellRect = shell.getClientArea();
				int right = shellRect.width - sashRect.width - limit;
				e.x = Math.max(Math.min(e.x, right), limit);
				if (e.x != sashRect.x) {
					sashData.left = new FormAttachment(0, e.x);
					contentComposite.layout();
				}
			}
		});
		EisMockerUtils.getLogger().exiting("EisMockerShell", "initSash");
	}

	public void setSupportedBOList(String[] boList) {
		this.supported_bo_list = boList;
	}

	public void setStatus(String text) {
		this.statusLabel.setText(text);
	}

	private void initReceivedListViewer(DataStore model) {

		// Set the content provider and label provider.
		receivedListViewer.setContentProvider(new BOListViewerProviders(EisMockerConstaints.RECEIVE_LIST));
		receivedListViewer.setLabelProvider(new BOListViewerProviders(EisMockerConstaints.RECEIVE_LIST));
		receivedListViewer.setInput(model);

		receivedListViewer.getList().addKeyListener(new BOListViewerKeyListener(EisMockerConstaints.RECEIVE_LIST));
		receivedListViewer.addSelectionChangedListener(new BOListViewerSelectionChangedListener(EisMockerConstaints.RECEIVE_LIST));
	}

	private void initSentListViewer(DataStore model) {
		sendListViewer.setContentProvider(new BOListViewerProviders(EisMockerConstaints.SEND_LIST));
		sendListViewer.setLabelProvider(new BOListViewerProviders(EisMockerConstaints.SEND_LIST));
		sendListViewer.setInput(model);

		sendListViewer.getList().addKeyListener(new BOListViewerKeyListener(EisMockerConstaints.SEND_LIST));
		sendListViewer.addSelectionChangedListener(new BOListViewerSelectionChangedListener(EisMockerConstaints.SEND_LIST));
	}

	protected class BOListViewerKeyListener implements KeyListener {
		private int actionType = -1;

		public BOListViewerKeyListener(int type) {
			actionType = type;
		}

		public void keyPressed(KeyEvent event) {
			if (event.keyCode == SWT.DEL) {
				if (actionType == EisMockerConstaints.SEND_LIST) {
					dataStore.removeEventDataItem(curSentListSelectedItem);
					sendListViewer.refresh();
					sdoTreeViewer.setContentSDO(null);
				} else {
					dataStore.removeRequestDataItem(curReceivedListSelectedItem);
					receivedListViewer.refresh();
					sdoTreeViewer.setContentSDO(null);
				}
			}
		}

		public void keyReleased(KeyEvent evt) {
		}
	}

	protected class BOListViewerSelectionChangedListener implements ISelectionChangedListener {
		private int actionType = -1;

		public BOListViewerSelectionChangedListener(int type) {
			actionType = type;
		}

		public void selectionChanged(SelectionChangedEvent event) {
			ISelection selection = event.getSelection();
			if (selection instanceof IStructuredSelection) {
				DataObject inputSDO = null;
				Object item = ((IStructuredSelection) selection).getFirstElement();
				if (item != null) {
					if (actionType == EisMockerConstaints.SEND_LIST) {
						inputSDO = ((EventDataItem) item).getDataObject();
						curSentListSelectedItem = (EventDataItem) item;
						curReceivedListSelectedItem = null;
					} else {
						inputSDO = ((RequestDataItem) item).getDataObject();
						curSentListSelectedItem = null;
						curReceivedListSelectedItem = (RequestDataItem) item;
					}
					sdoTreeViewer.setContentSDO(inputSDO);
					updateCurrentBOType(inputSDO);
				}
			}
		}
	}

	public void actionNewBO() {
		try {
			String boType = boTypeCombo.getText();
			if (boType == null || boType.equalsIgnoreCase("")) {
				showMessageBox("Warning", "Please choose one Business Object type first.");
				return;
			}
			String nameSpace = XSDScanner.getNameSpaceByName(boType);
			DataObject dataObj = DataFactory.INSTANCE.create(nameSpace, boType);
			sdoTreeViewer.setContentSDO(dataObj);
			curSentListSelectedItem = null;
			curReceivedListSelectedItem = null;
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			showMessageBox("Error", "Failed to create Business Object instance");

		}
	}

	public void actionSendBO() {
		DataObject dataObject = sdoTreeViewer.getContentSDO();
		if (dataObject == null) {
			showMessageBox("Warning", "You must create or choose one Business Object instance before using this function.");
			return;
		}
		if (dataObject != null && dataStore != null) {
			DataObject newDataObj = CopyHelper.INSTANCE.copy(dataObject);
			String operation = opTypeCombo.getText();
			dataStore.addEventDataItem(newDataObj, operation);
			//XMLHelper.INSTANCE.
//			System.out.println(newDataObj.getType().getURI());
//			String strtmp = commonj.sdo.helper.XMLHelper.INSTANCE.save(newDataObj, newDataObj.getType().getURI(), newDataObj.getType().getName());
//			//strtmp = strtmp.replace("&lt;", "<");
//			//strtmp = "[abcd]";
//			//strtmp = strtmp.replace("&lt;", "<");
//			strtmp = strtmp.replace('<', '[');
//			strtmp = strtmp.replace('>', ']');			
//			System.out.println(strtmp);
//			EisMockerUtils.getLogger().log(Level.FINE,"\n"+strtmp+"\n");
		} else {
			EisMockerUtils.getLogger().log(Level.FINER, "Failed to send out the Business Object instance");
		}
	}

	public void actionSendBOInBatch() {
		DataObject sdo = sdoTreeViewer.getContentSDO();
		if (sdo == null) {
			showMessageBox("Warning", "You must create or choose one Business Object instance before using this function.");
			return;
		}
		String operation = opTypeCombo.getText();
		BatchSendBOShell dialog = new BatchSendBOShell(shell, sdo, operation);
		dialog.open();

	}

	public void actioinRemoveBO() {
		if (curSentListSelectedItem == null && curReceivedListSelectedItem == null) {
			showMessageBox("Warning", "You didn't select any Business Object instance to delete");
			return;
		}
		if (curSentListSelectedItem != null) {
			dataStore.removeEventDataItem(curSentListSelectedItem);
			sendListViewer.refresh();
			sdoTreeViewer.setContentSDO(null);
			curSentListSelectedItem = null;
		} else if (curReceivedListSelectedItem != null) {
			dataStore.removeRequestDataItem(curReceivedListSelectedItem);
			receivedListViewer.refresh();
			sdoTreeViewer.setContentSDO(null);
			curReceivedListSelectedItem = null;
		}
	}

	public void actionClearList() {
		CTabItem item = cFolder.getSelection();
		String tabText = item.getText();
		if (tabText.equals(EisMockerConstaints.RECEIVE_TAB)) {
			if (dataStore.getRequestDataItemCount() == 0) {
				showMessageBox("Warning", "The current Busniess Object instance list is empty.");
			} else {
				dataStore.clearRequestDataItemList();
				receivedListViewer.setSelection(null);
				receivedListViewer.refresh();
				curReceivedListSelectedItem = null;
				sdoTreeViewer.setContentSDO(null);
			}
		} else if (tabText.equals(EisMockerConstaints.SEND_TAB)) {
			if (dataStore.getEventDataItemCount() == 0) {
				showMessageBox("Warning", "The current Busniess Object instance list is empty.");
			} else {
				dataStore.clearEventDataItemList();
				sendListViewer.setSelection(null);
				sendListViewer.refresh();
				curSentListSelectedItem = null;
				sdoTreeViewer.setContentSDO(null);
			}
		}
	}

	public void actionSaveBO() {
		try {
			DataObject dataObject = sdoTreeViewer.getContentSDO();
			if(dataObject == null){
				showMessageBox("Warning", "Please select or create one Business Object instance to save.");
				return;
			}
			FileDialog fileDlg = new FileDialog(shell, SWT.SAVE);
			fileDlg.setFilterExtensions(new String[] { "*.xml" });
			String filePath = fileDlg.open();
			if (filePath != null) {
				FileOutputStream outfile = new FileOutputStream(filePath);
				String typeName = dataObject.getType().getName();
				String nameSpace = XSDScanner.getNameSpaceByName(typeName);
				XMLHelper.INSTANCE.save(dataObject, nameSpace, typeName, outfile);
				outfile.close();
			}
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			showMessageBox("Warning", "Failed to save the Business Object instance.");
		}
	}

	public void actionLoadBO() {
		FileDialog fileDlg = new FileDialog(shell, SWT.OPEN);
		fileDlg.setFilterExtensions(new String[] { "*.xml" });
		String fileName = fileDlg.open();
		if (fileName != null) {
			try {
				FileInputStream inputfile = new FileInputStream(fileName);
				DataObject inputSDO = XMLHelper.INSTANCE.load(inputfile).getRootObject();
				sdoTreeViewer.setContentSDO(inputSDO);
				updateCurrentBOType(inputSDO);
			} catch (Exception e) {
				EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
				showMessageBox("Warning", "Failed to load the Business Object instance.");
			}
		}
	}

	public void actionReplyBO() {
		RequestDataItem dataItem = curReceivedListSelectedItem;

		if (dataItem == null) {
			showMessageBox("Warning", "Please select one Business Object instance from the Receive list to reply.");
		} else if (dataItem.getIsReplied()) {
			showMessageBox("Warning", "The selected Business Object instance has been replied before.");
		} else {
			ResponseBOShell responseShell = new ResponseBOShell(shell, dataItem);
			responseShell.open();
		}
	}

	public void actionFileExit() {
		shell.close();
	}

	public void actionHelpAbout() {
		MessageBox messageBox = new MessageBox(shell, SWT.ICON_INFORMATION);
		messageBox.setMessage("EIS Mocker");
		messageBox.open();
	}

	public void receivedNewItems() {
		if (!display.isDisposed()) {
			display.asyncExec(new Runnable() {
				public void run() {
					receivedListViewer.refresh();
					statusLabel.setText("Received List Changed.");
				}
			});
		}
	}

	public void sendListChanged() {
		if (!display.isDisposed()) {
			display.asyncExec(new Runnable() {
				public void run() {
					sendListViewer.refresh();
					setStatus("Sent List Changed.");
				}
			});
		}
	}

	public void showMessageBox(String type, String msg) {
		MessageBox messageBox = new MessageBox(shell, SWT.ICON_WARNING);
		messageBox.setText(type);
		messageBox.setMessage(msg);
		messageBox.open();
	}

	public void updateCurrentBOType(DataObject sdo) {
		String boName = sdo.getType().getName();
		int curIdx = -1;
		for (curIdx = supported_bo_list.length - 1; curIdx > -1; curIdx--) {
			if (boName.equals(supported_bo_list[curIdx])) {
				break;
			}
		}
		if (curIdx != -1) {
			boTypeCombo.select(curIdx);
		}

	}
}
