/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

/*
 * BTreeItem.java - 2007-10-21
 * Rongda LIANG @IBM
 */
package com.ibm.eismocker.ui.batch;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import com.ibm.eismocker.main.EisMockerConstaints;

import commonj.sdo.DataObject;
import commonj.sdo.Property;

/**
 * A light encapsulation for TreeViewer of BatchShell.
 */
public class BTreeItem implements Serializable{
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}
	private static final long serialVersionUID = 123L;
	private Object object;
	private Property property;
	private BTreeItem parent;
	//The batch and the corresponding value.
	//private RuleEnum_delete rule;
	private String rule;
	private String value;
	//List use to hold the children.
	private ArrayList<BTreeItem> childList;	
	private boolean checkedChild = false; 
	//Indicate the Item is wrapper or not. For isMany() == true property.
	//The program will add a new wrapper to them. 
	private boolean isWrapper;
	
	public BTreeItem(Object object, Property property,
			BTreeItem parent, boolean isMany) {
		this.object = object;
		this.property = property;
		this.parent = parent;
		this.rule = EisMockerConstaints.SUPPORT_RULE_NONE;
		this.isWrapper = isMany;
	}
	
	public Object getObject() {
		return object;
	}

	public Property getProperty() {
		return property;
	}

	public String getRule() {
		return rule;
	}
	
	public boolean isWrapper() {
		return isWrapper;
	}

	public void setRule(String rule) {
		this.rule = rule;
	}

	public BTreeItem getParent() {
		return parent;
	}

	public boolean isChildChecked() {
		return checkedChild;
	}

	public void setChildChecked(boolean isChildChecked) {
		this.checkedChild = isChildChecked;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public void setParent(BTreeItem parent) {
		this.parent = parent;
	}	
	
	public boolean isDataObjectNode(){
		if(object instanceof DataObject){
			return true;
		}
		return false;
	}
	
	public boolean add(BTreeItem item) {
		if(childList == null)
			childList = new ArrayList<BTreeItem>();
		item.setParent(this);
		return childList.add(item);
	}

	public void addAll(Collection<BTreeItem> c) {		
		if(childList == null)//For wrapper without any child.
			childList = new ArrayList<BTreeItem>();
		for (Iterator<BTreeItem> iterator = c.iterator(); iterator.hasNext();) {
			BTreeItem item = (BTreeItem) iterator.next();
			add(item);
		}
	}

	public BTreeItem[] getChildren() {
		return childList.toArray(new BTreeItem[0]);
	}

	public boolean hasChildren(){
		return getChildrenCounts() > 0;
	}
	
	public int getChildrenCounts(){
		if(childList == null)
			return 0;
		return childList.size();
	}
	
	public boolean remove(BTreeItem obj) {
		return childList.remove(obj);
	}

	public int getIndex(BTreeItem obj) {
		return childList.indexOf(obj);
	}
	

}
