/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.ui.batch;

import java.math.BigInteger;
import java.util.List;
import java.util.logging.Logger;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ICellEditorListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolTip;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;

import com.ibm.eismocker.datastore.DataStore;
import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.utils.EisMockerUtils;
import com.ibm.eismocker.utils.ImageManager;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.helper.CopyHelper;

public class BatchSendBOShell {
	static String copyright() {
		return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private final Shell shell;

	private TreeViewer treeViewer;

	private DataObject dataObj;

	private Logger logger = EisMockerUtils.getLogger();

	private String operation;

	private BTreeItem curTreeItem = null;

	private Text txTime = null;

	public BatchSendBOShell(Shell parent, DataObject dataObj, String operation) {
		this.dataObj = dataObj;
		this.operation = operation;

		shell = new Shell(parent, SWT.SHELL_TRIM | SWT.ON_TOP | SWT.APPLICATION_MODAL);
		shell.setImage(ImageManager.get("MainIcon"));
		shell.setText("Batch Send");
		shell.setSize(400, 400);
		shell.setLayout(new GridLayout());
		shell.setLocation(parent.getLocation().x + 100, parent.getLocation().y + 30);

		initTreeViewer();
		initButtonComposite();
	}

	private void initButtonComposite() {
		Composite bottomComposite = new Composite(shell, SWT.FILL);
		GridLayout cLayout = new GridLayout(4, false);
		bottomComposite.setLayout(cLayout);

		// Time text field
		Label label = new Label(bottomComposite, SWT.NONE);
		label.setText("Times :");
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.END;
		label.setLayoutData(gridData);

		txTime = new Text(bottomComposite, SWT.NONE);
		gridData = new GridData(GridData.FILL_BOTH);
		txTime.setLayoutData(gridData);

		// Send Button
		Button sendButton = new Button(bottomComposite, SWT.NONE);
		sendButton.setText("&Send");
		sendButton.addSelectionListener(new BatchSendSelectionListener(this, EisMockerConstaints.BATCH_ACTION_SEND));
		gridData = new GridData(GridData.FILL_BOTH);
		sendButton.setLayoutData(gridData);

		// Cancel Button
		Button cancelButton = new Button(bottomComposite, SWT.NONE);
		cancelButton.setText("C&ancel");
		cancelButton.addSelectionListener(new BatchSendSelectionListener(this, EisMockerConstaints.BATCH_ACTION_CANCEL));
		gridData = new GridData(GridData.FILL_BOTH);
		cancelButton.setLayoutData(gridData);
	}

	public void actionSend() {

		if (treeViewer == null)
			return;

		// Check time input box.
		int times = -1;
		try {
			times = Integer.parseInt(txTime.getText());
		} catch (Exception e) {
			showMessageBox("The value for Times is invalid!");
			return;
		}
		if (times <= 0) {
			showMessageBox("The value for Times must be positive integer!");
			return;
		}

		BTreeItem treeItem = (BTreeItem) treeViewer.getInput();

		// Check value fields.
		if (!validateRuleValue(treeItem)) {
			return;
		}

		DataObject dataObj = (DataObject) treeItem.getObject();
		final DataObject copyObj = CopyHelper.INSTANCE.copy(dataObj);

		// Clear the items whose rule value is empty.
		final BTreeItem reducedTreeItem = clearEmptyRules(treeItem);
		final int sendTimes = times;
		Thread batchThread = new Thread(new Runnable() {
			private DataStore model = DataStore.getDataStore();
			public void run() {
				StringMaker.reset();
				implementSizeRule(reducedTreeItem, copyObj);
				for (int i = 0; i < sendTimes; i++) {
					varyItemValue(reducedTreeItem, i, copyObj);
					model.addEventDataItem(CopyHelper.INSTANCE.copy(copyObj), operation);
				}
			}

		});
		batchThread.start();
	}

	public void actionCancel() {
		dataObj = null;
		shell.dispose();
	}

	private boolean validateRuleValue(BTreeItem item) {
		if (item.hasChildren()) {
			BTreeItem[] treeItems = item.getChildren();
			for (int idx = 0; idx < treeItems.length; idx++) {
				if (!validateRuleValue(treeItems[idx]))
					return false;
			}
		} else {
			if ((!item.isWrapper()) && (!item.isDataObjectNode())) {
				if (!EisMockerConstaints.SUPPORT_RULE_NONE.equals(item.getRule())) {
					if ((item.getValue() == null) || (item.getValue().equals(""))) {
						Property property = item.getProperty();
						showMessageBox("The rule value for Property " + property.getName() + " can't be empty!");
						return false;
					}
				}
			}
		}
		return true;
	}

	/**
	 * Implement of size rule for specified DataObject.
	 * 
	 * @param item
	 * @param dataObj
	 */
	private void implementSizeRule(BTreeItem item, DataObject dataObj) {
		implementSizeRule0(item, dataObj, true, -1);
	}

	/**
	 * Inner method for <code>implementSizeRule</code>.
	 * 
	 * @param item
	 * @param parentObj
	 * @param isFirst
	 * @param index
	 *            Only use when the <code>isMany</code> method of Property
	 *            returns true. Otherwise, set to -1.
	 */
	private void implementSizeRule0(BTreeItem item, Object parentObj, boolean isFirst, int index) {
		if(item == null)
			return;
		Property property = item.getProperty();
		if (item.hasChildren()) {
			Object curParentObj;
			boolean isMany = false;
			if (isFirst) {
				curParentObj = parentObj;
			} else {
				BTreeItem pItem = item.getParent();
				isMany = property.isMany();
				if (pItem != null && pItem.isWrapper()) {
					curParentObj = ((List<?>) parentObj).get(index);
				} else {
					curParentObj = ((DataObject) parentObj).get(item.getProperty());
				}
			}
			BTreeItem[] treeItems = item.getChildren();
			for (int i = 0; i < treeItems.length; i++) {
				if (isMany) {
					implementSizeRule0(treeItems[i], curParentObj, false, i);
				} else {
					implementSizeRule0(treeItems[i], curParentObj, false, -1);
				}
			}
		}
		if (!item.isDataObjectNode() && !item.isWrapper()) {
			if (item.getRule().equals(EisMockerConstaints.SUPPORT_RULE_SIZE)) {
				String value = item.getValue().toLowerCase().trim();
				int size;
				if (value.endsWith("kb")) {
					size = Integer.parseInt(value.substring(0, value.length() - 2));
					size *= 1024;
				} else if (value.endsWith("mb")) {
					size = Integer.parseInt(value.substring(0, value.length() - 2));
					size *= 1024 * 1024;
				} else { // End with "b"
					size = Integer.parseInt(value.substring(0, value.length() - 1));
				}
				String str = StringMaker.buildString(size);

				// Set New Value.
				BTreeItem parentItem = item.getParent();

				DataObject dataObj = (DataObject) parentObj;
				dataObj.set(property, str);
				// Erase this node after setting done.
				parentItem.remove(item);
				// Write log.
				logger.finest(property.getName() + " is set to " + size + "bytes.");
			}
		}
	}

	/**
	 * Inner method use to vary the value of the treeItem according to its rule
	 * to a targetObj.
	 * 
	 * @param item
	 * @param curIndex
	 * @param dataObj
	 */
	private void varyItemValue(BTreeItem item, int curIndex, DataObject dataObj) {
		varyItemValue0(item, curIndex, dataObj, true, -1);
	}

	// Parameters is similar to <code>implementSizeRule0</code>
	private void varyItemValue0(BTreeItem item, int curIndex, Object parentObj, boolean isFirst, int index) {
		if(item == null)
			return;
		
		Property property = item.getProperty();
		if (item.hasChildren()) {
			Object curParentObj;
			boolean isMany = false;
			if (isFirst) {
				curParentObj = parentObj;
			} else {
				BTreeItem pItem = item.getParent();
				isMany = property.isMany();
				if (pItem != null && pItem.isWrapper()) {
					curParentObj = ((List<?>) parentObj).get(index);
				} else {
					curParentObj = ((DataObject) parentObj).get(item.getProperty());
				}
			}
			BTreeItem[] treeItems = item.getChildren();
			for (int i = 0; i < treeItems.length; i++) {
				if (isMany) {
					varyItemValue0(treeItems[i], curIndex, curParentObj, false, i);
				} else {
					varyItemValue0(treeItems[i], curIndex, curParentObj, false, -1);
				}
			}
		}

		if (!item.isDataObjectNode() && !item.isWrapper()) {
			// Get value settings.
			// Object obj = item.getObject();

			String value = item.getValue();
			int initialValue = Integer.parseInt(value.substring(0, value.indexOf(',')));
			int step = Integer.parseInt(value.substring(value.indexOf(',') + 1, value.length()));
			// Build new value.
			int intValue;
			if (item.getRule().equals(EisMockerConstaints.SUPPORT_RULE_INC)) {
				intValue = initialValue + step * curIndex;
			} else if (item.getRule().equals(EisMockerConstaints.SUPPORT_RULE_DEC)) {
				intValue = initialValue - step * curIndex;
			} else {
				return;
			}

			String baseTypeName = EisMockerUtils.getBaseTypeName(property.getType());
			Object newValue = null;
			if (baseTypeName.equals("String")) {
				newValue = Integer.toString(intValue);
			} else if (baseTypeName.equals("Int")) {
				newValue = intValue;
			} else if (baseTypeName.equals("Long")) {
				newValue = intValue;
			} else if (baseTypeName.equals("Byte")) {
				newValue = intValue;
			} else if (baseTypeName.equals("Integer")) {
				newValue = new BigInteger(Integer.toString(intValue));
			}
			DataObject dataObj = (DataObject) parentObj;
			dataObj.set(item.getProperty(), newValue);
			// Write log.
			logger.finest("Times " + curIndex + " " + property.getName() + " Changed to " + newValue);
		}
	}

	/**
	 * Make a shallow copy of BTreeItem only contains nodes which have rules.
	 * 
	 * @param item
	 *            input object.
	 * @return a reduced TreeItem
	 */
	private BTreeItem clearEmptyRules(BTreeItem item) {
		BTreeItem newItem = null;
		// Copy nodes recursively.
		if (item.hasChildren()) {
			BTreeItem[] treeItems = item.getChildren();
			for (int idx = 0; idx < treeItems.length; idx++) {
				BTreeItem childItem = clearEmptyRules(treeItems[idx]);
				if (childItem != null) {
					if (newItem == null) {
						newItem = shallowCopy(item);
					}
					newItem.add(childItem);
				}
			}
		} else if (!item.getRule().equals(EisMockerConstaints.SUPPORT_RULE_NONE)) {
			newItem = shallowCopy(item);
		}
		return newItem;
	}

	private BTreeItem shallowCopy(BTreeItem item) {
		BTreeItem copyItem = new BTreeItem(item.getObject(), item.getProperty(), null, item.isWrapper());
		copyItem.setRule(item.getRule());
		copyItem.setValue(item.getValue());
		return copyItem;
	}

	private void initTreeViewer() {
		treeViewer = new TreeViewer(shell, SWT.FULL_SELECTION | SWT.BORDER);
		// Set Layout.
		GridData gridData = new GridData(GridData.FILL_BOTH);
		Tree tree = treeViewer.getTree();
		tree.setLayoutData(gridData);

		tree.setHeaderVisible(true);
		tree.setLinesVisible(true);

		// Initialize treeViewer columns.
		treeViewer.setColumnProperties(new String[] { EisMockerConstaints.NAME, EisMockerConstaints.RULE, EisMockerConstaints.VALUE });
		createColumn(tree, EisMockerConstaints.NAME, 176, SWT.CENTER);
		createColumn(tree, EisMockerConstaints.RULE, 80, SWT.CENTER);
		createColumn(tree, EisMockerConstaints.VALUE, 100, SWT.CENTER);

		treeViewer.setContentProvider(new BatchSendTreeContentProvider());
		treeViewer.setLabelProvider(new BatchSendTreeLabelProvider());
		treeViewer.setCellModifier(new BatchSendTreeCellModifier(shell, treeViewer));

		// Initialize editor.
		CellEditor[] editors = new CellEditor[3];
		editors[1] = new ComboBoxCellEditor(tree, EisMockerConstaints.SUPPORT_VALUE_RULES, SWT.READ_ONLY);

		final ComboBoxCellEditor cEditor = (ComboBoxCellEditor) editors[1];
		// Build the ToolTip block.
		final ToolTip tip = new ToolTip(shell, SWT.BALLOON | SWT.ICON_INFORMATION);
		tip.setText("Rule Format");

		cEditor.addListener(new ICellEditorListener() {
			// Fix the position in treeViewer. May be varied in other system.
			private static final int X = 9;

			private static final int Y = 55;

			public void applyEditorValue() {
				Integer selection = (Integer) cEditor.getValue();
				Point point = shell.getLocation();
				Rectangle r = cEditor.getControl().getBounds();
				if (curTreeItem == null) {
					return;
				}
				switch (selection) {
				case 0: // Selection is "None".
					// Clear the value in this tree Item.
					curTreeItem.setValue(null);
					break;
				case 1:// "Increase"
					curTreeItem.setValue("0,2");
					tip.setMessage("Initial value, Step.\ne.g. 0, 2 => 0, 2, 4....");
					break;
				case 2:// "Decrease"
					curTreeItem.setValue("100,2");
					tip.setMessage("Initial value, Step.\ne.g. 100, 2 => 100, 98, 96....");
					break;
				case 3:// "Size"
					curTreeItem.setValue("1kB");
					tip.setMessage("size (b|kb|mb)\ne.g. 1kb");
					break;
				default:
					logger.severe("Selection error.");
					break;
				}
				if (selection != 0) {
					// Show Tooltips.
					tip.setLocation(X + point.x + r.x + (int) (r.width * 1.5), Y + point.y + r.y + r.height);
					tip.setVisible(true);
				}
			}

			public void cancelEditor() {
			}

			public void editorValueChanged(boolean arg0, boolean arg1) {
			}
		});

		editors[2] = new TextCellEditor(tree);
		treeViewer.setCellEditors(editors);
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent evt) {
				// TODO Auto-generated method stub
				ISelection selection = evt.getSelection();
				if (selection instanceof IStructuredSelection) {
					IStructuredSelection structuredSelection = (IStructuredSelection) selection;
					curTreeItem = (BTreeItem) structuredSelection.getFirstElement();
					// logger.finest("Item " + curTreeItem + " selected.");
				}
			}
		});
		treeViewer.setInput(new BTreeItem(dataObj, null, null, false));
		treeViewer.expandToLevel(3);
	}

	private TreeColumn createColumn(Tree tree, String label, int width, int style) {
		TreeColumn column = new TreeColumn(tree, style);
		column.setText(label);
		column.setWidth(width);
		return column;
	}

	public void open() {
		shell.open();
	}

	private void showMessageBox(String message) {
		MessageBox messageBox = new MessageBox(shell, SWT.ICON_WARNING | SWT.YES);
		messageBox.setMessage(message);
		messageBox.open();
	}
}
