/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.eismocker.ui.batch;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;

import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.utils.EisMockerUtils;
import commonj.sdo.Property;

public class BatchSendTreeCellModifier implements ICellModifier {
	static String copyright() {
		return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private Shell shell;

	private TreeViewer treeViewer;

	public BatchSendTreeCellModifier(Shell shell, TreeViewer viewer) {
		this.shell = shell;
		treeViewer = viewer;
	}

	public boolean canModify(Object element, String col) {
		BTreeItem treeItem = (BTreeItem) element;
		// Only accept Rule and Value Field.
		if (EisMockerConstaints.NAME.equals(col)) {
			return false;
		}

		if (EisMockerConstaints.RULE.equals(col)) {
			CellEditor[] editors = treeViewer.getCellEditors();
			Property property = (Property) treeItem.getProperty();
			if (property != null) {
				String baseTypeName = EisMockerUtils.getBaseTypeName(property.getType());
				ComboBoxCellEditor editor = (ComboBoxCellEditor) editors[1];
				//CR 91881 - the rule for "verb" should be "None".
				if(property.getName().equals("verb")){
					editor.setItems(new String[] {EisMockerConstaints.SUPPORT_RULE_NONE});
				}
				else if (baseTypeName.equals("String")) {
					editor.setItems(EisMockerConstaints.SUPPORT_VALUE_RULES);
				} else {
					editor.setItems(new String[] { EisMockerConstaints.SUPPORT_RULE_NONE, EisMockerConstaints.SUPPORT_RULE_INC, EisMockerConstaints.SUPPORT_RULE_DEC });
				}
			}
		}

		if (!(treeItem.isWrapper() || treeItem.isDataObjectNode())) {
			String typeName = treeItem.getProperty().getType().getName();
			// Batch function not for Boolean type yet.
			if (!typeName.equalsIgnoreCase(EisMockerConstaints.BOOLEAN)) {
				if (EisMockerConstaints.VALUE.equals(col)) {
					if (treeItem.getRule().equals(EisMockerConstaints.SUPPORT_RULE_NONE)) {
						// None Rule, value field cannot be edited.
						return false;
					}
				}
				return true;
			}
		}
		return false;
	}

	public Object getValue(Object element, String col) {
		BTreeItem treeItem = (BTreeItem) element;
		if (EisMockerConstaints.RULE.equals(col)) {
			String rule = treeItem.getRule();
			return EisMockerUtils.getRuleIndexByName(rule);
		}
		if (EisMockerConstaints.VALUE.equals(col)) {
			String value = treeItem.getValue();
			if (value == null) {
				value = "";
			}
			return value;
		}
		return null;
	}

	public void modify(Object element, String col, Object value) {
		TreeItem item = (TreeItem) element;
		BTreeItem treeItem = (BTreeItem) item.getData();

		if (EisMockerConstaints.RULE.equals(col)) {
			int index = ((Integer) value).intValue();
			treeItem.setRule(EisMockerUtils.getRuleNameByIndex(index));
			
		} else if (EisMockerConstaints.VALUE.equals(col)) {
			String strVal = ((String) value).toLowerCase();

			// Check the input validate.
			String rule = treeItem.getRule();
			if (!validate(strVal, rule)) {
				MessageBox messageBox = new MessageBox(shell, SWT.ICON_WARNING | SWT.YES);
				String message = "\"" + strVal + "\" is an invalid value.";
				if (rule.equals(EisMockerConstaints.SUPPORT_RULE_SIZE)) {
					message += "\nFormat: number + [b|kb|mb]. e.g. 10kb";
				} else {
					message += "\nFormat: (Initial value),(variation) e.g. 100,2";
				}
				messageBox.setMessage(message);
				messageBox.open();
				return;
			}
			treeItem.setValue((String) value);
		}
		// Force refresh viewer.
		treeViewer.refresh();
	}

	private boolean validate(String value, String rule) {
		if (rule.equals(EisMockerConstaints.SUPPORT_RULE_INC) || rule.equals(EisMockerConstaints.SUPPORT_RULE_DEC)) {
			if (value.matches("\\d+,\\d+")) {
				return true;
			}
		} else if (rule.equals(EisMockerConstaints.SUPPORT_RULE_SIZE)) {
			if (value.matches("\\d+[km]{0,1}b{1}")) {
				return true;
			}
		}
		return false;
	}
}
