/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.eismocker.ui.batch;

import java.util.List;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.utils.EisMockerUtils;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

public class BatchSendTreeContentProvider implements ITreeContentProvider {
	static String copyright() {
		return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	@SuppressWarnings("unchecked")
	public Object[] getChildren(Object item) {
		BTreeItem treeItem = (BTreeItem) item;

		if (treeItem.isChildChecked()) {
			return treeItem.getChildren();
		}
		// Normal Property, return.
		if (!treeItem.isDataObjectNode()) {
			return null;
		}

		DataObject dataObj = (DataObject) treeItem.getObject();
		Type type = dataObj.getType();
		List<Property> list = type.getProperties();
		for (int i = 0; i < list.size(); i++) {
			Property property = list.get(i);
			// Batch Function only for String properties
			Type curType = property.getType();
			String typeName = curType.getName();
			// Get base Type if base Type availiable.
			typeName = EisMockerUtils.getBaseTypeName(curType);

			if (isSupportedTypeName(typeName) || property.isContainment()) {
				if (property.isMany()) {
					BTreeItem containerItem = new BTreeItem(null, property, treeItem, true);
					List<?> values = dataObj.getList(i);
					// All all children items to the container.
					for (int j = 0; j < values.size(); j++) {
						BTreeItem childItem = new BTreeItem(values.get(j), property, treeItem, false);
						containerItem.add(childItem);
					}
					containerItem.setChildChecked(true);
					treeItem.add(containerItem);
				} else {
					treeItem.add(new BTreeItem(dataObj.get(i), property, treeItem, false));
				}
			}
		}
		// Mark treeItem node has been checked.
		treeItem.setChildChecked(true);
		return treeItem.getChildren();
	}

	private boolean isSupportedTypeName(String name) {
		if (name != null) {
			for (int i = 0; i < EisMockerConstaints.SUPPORT_DATA_TYPES.length; i++) {
				if (name.equals(EisMockerConstaints.SUPPORT_DATA_TYPES[i])) {
					return true;
				}
			}
		}
		return false;
	}

	public Object getParent(Object obj) {
		BTreeItem treeItem = (BTreeItem) obj;
		return treeItem.getParent();
	}

	public boolean hasChildren(Object obj) {
		BTreeItem treeItem = (BTreeItem) obj;

		if (treeItem.isChildChecked()) {
			return treeItem.hasChildren();
		}

		if (!treeItem.isDataObjectNode()) {
			return false;
		}

		DataObject dataObj = (DataObject) treeItem.getObject();
		List<?> list = dataObj.getType().getProperties();
		return (list.size() > 0);
	}

	public Object[] getElements(Object obj) {
		return getChildren(obj);
	}

	public void dispose() {
	}

	public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
	}

}
