/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.utils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ibm.eismocker.main.EisMockerConstaints;
import com.ibm.eismocker.rmidriver.DataMap;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;

public class EisMockerUtils {
	static String copyright() {
		return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}

	// Log message of the program.
	private static Logger logUtils = null;

	private static String logFileName = "";

	@SuppressWarnings("unchecked")
	// ///////////////////////////////////////////////////////////////////////////////
	// get the base type
	public static String getBaseTypeName(Type type) {
		List<Type> baseTypeList = type.getBaseTypes();
		if (baseTypeList.size() != 0) {
			type = baseTypeList.get(0);
		}
		return type.getName();
	}

	// ///////////////////////////////////////////////////////////////////////////////
	// Convert SDO to DataMap
	public static DataMap sdo2DataMap(DataObject dataObj) {
		String boName = dataObj.getType().getName();
		DataMap dataMap = new DataMap(boName);
		copySDO2DataMap(dataObj, dataMap);
		return dataMap;
	}

	@SuppressWarnings("unchecked")
	private static void copySDO2DataMap(DataObject dataObj, DataMap dataMap) {
		Iterator iterator = dataObj.getType().getProperties().iterator();
		while (iterator.hasNext()) {
			Property prop = (Property) iterator.next();
			if (dataObj.isSet(prop)) {
				if (prop.isContainment()) { // is child BO
					if (prop.isMany()) {// N-Card
						List childList = dataObj.getList(prop);
						for (int idx = 0; idx < childList.size(); idx++) {
							DataMap dataMapChild = dataMap.createDataMap(prop.getName(), true);
							copySDO2DataMap((DataObject) childList.get(idx), dataMapChild);
						}
					} else {// 1-Card
						DataMap dataMapChild = dataMap.createDataMap(prop.getName(), false);
						copySDO2DataMap(dataObj.getDataObject(prop), dataMapChild);
					}
				} else {// simple type
					if (prop.isMany()) {// N-Card
						List valueList = dataObj.getList(prop);
						ArrayList newList = new ArrayList();
						for (int idx = 0; idx < valueList.size(); idx++) {
							newList.add(valueList.get(idx));
						}
						dataMap.set(prop.getName(), newList);
					} else {// 1-Card
						Object value = dataObj.get(prop);
						dataMap.set(prop.getName(), value);
					}
				}
			}
		}
	}

	// ///////////////////////////////////////////////////////////////////////////////
	// Convert DataMap to SDO
	public static DataObject dataMap2Sdo(DataMap dataMap) {

		DataObject dataObj = null;
		try {
			String typeName = dataMap.getName();
			String nameSpace = XSDScanner.getNameSpaceByName(typeName);
			dataObj = DataFactory.INSTANCE.create(nameSpace, typeName);
			copyDataMap2SDO(dataMap, dataObj);
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
		}
		return dataObj;
	}

	private static void copyDataMap2SDO(DataMap dataMap, DataObject dataObj) {
		Type metadata = dataObj.getType();
		Iterator iterator = dataMap.keySet().iterator();
		while (iterator.hasNext()) {
			String propName = (String) iterator.next();
			Property prop = metadata.getProperty(propName);
			if (prop.isContainment()) {// is child BO
				if (prop.isMany()) {// N-Card
					List childList = dataMap.getList(propName);
					for (int idx = 0; idx < childList.size(); idx++) {
						DataObject childDataObj = dataObj.createDataObject(prop);
						copyDataMap2SDO((DataMap) childList.get(idx), childDataObj);
					}
				} else {// 1-Card
					DataObject childDataObj = dataObj.createDataObject(prop);
					copyDataMap2SDO(dataMap.getDataMap(propName), childDataObj);
				}
			} else {// is simple type
				if (prop.isMany()) {// N-Card
					//List valueList = dataMap.getList(propName);
					List valueList = (List)dataMap.get(propName);
					dataObj.setList(prop, valueList);
				} else {// 1-Card
					Object value = dataMap.get(propName);
					//CR92036 - Convert the date/dateTime to correct String format.
					if (value instanceof java.util.Date) {
						if(prop.getType().getName().equalsIgnoreCase("Date") || prop.getType().getName().equalsIgnoreCase("YearMonthDay")){
							SimpleDateFormat sdf = new SimpleDateFormat(EisMockerConstaints.DATEFORMAT_DATE);
							value = sdf.format(value);
						}else if(prop.getType().getName().equalsIgnoreCase("DateTime")){
							SimpleDateFormat sdf = new SimpleDateFormat(EisMockerConstaints.DATEFORMAT_DATETIME);
							value = sdf.format(value);							
						}else{
							value = value.toString();
						}
						//value = value.toString();
					}
					dataObj.set(prop, value);
				}
			}
		}
	}

	// ////////////////////////////////////////////////////////////////////////////////
	// get the logUtils.
	public static Logger getLogger() {
		if (logUtils == null) {
			logUtils = Logger.getLogger("com.ibm.eismoker.ui");
			try {
				// Handler handler = new ConsoleHandler();
				FileHandler handler = new FileHandler(logFileName);
				handler.setLevel(Level.ALL);
				logUtils.addHandler(handler);
			} catch (Exception e) {
				System.out.println("Error happens when generating the log file");
				// e.printStackTrace();
			}
		}
		return logUtils;
	}

	public static void setLogFileName(String fileName) {
		logFileName = fileName;
	}

	// ////////////////////////////////////////////////////////////////////////////////
	// map between the rule index and rule name
	public static int getRuleIndexByName(String ruleName) {
		int index = -1;
		if (ruleName.endsWith(EisMockerConstaints.SUPPORT_RULE_NONE)) {
			index = EisMockerConstaints.INDEX_RULE_NONE;
		}else if (ruleName.endsWith(EisMockerConstaints.SUPPORT_RULE_INC)) {
			index = EisMockerConstaints.INDEX_RULE_INC;
		}else if (ruleName.endsWith(EisMockerConstaints.SUPPORT_RULE_DEC)) {
			index = EisMockerConstaints.INDEX_RULE_DEC;
		}else if (ruleName.endsWith(EisMockerConstaints.SUPPORT_RULE_SIZE)) {
			index = EisMockerConstaints.INDEX_RULE_SIZE;
		}
		return index;
	}
	public static String getRuleNameByIndex(int ruleIdx) {
		String ruleName ="";
		switch(ruleIdx){
		case EisMockerConstaints.INDEX_RULE_NONE:
			ruleName = EisMockerConstaints.SUPPORT_RULE_NONE;
			break;
		case EisMockerConstaints.INDEX_RULE_INC:
			ruleName = EisMockerConstaints.SUPPORT_RULE_INC;
			break;
		case EisMockerConstaints.INDEX_RULE_DEC:
			ruleName = EisMockerConstaints.SUPPORT_RULE_DEC;
			break;
		case EisMockerConstaints.INDEX_RULE_SIZE:
			ruleName = EisMockerConstaints.SUPPORT_RULE_SIZE;
			break;
		}
		return ruleName;
		
	}
	
}
