/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.eismocker.utils;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.logging.Level;


public class PropertiesUtil {
	static String copyright() {
		 return com.ibm.eismocker.rmidriver.Copyright.IBM_COPYRIGHT_SHORT;
	}
	//File path.
	private String path = null;
	
	public static final int ADAPTER = 1;
	public static final int EIS = 2;
	
	//For EIS GUI.
	private static final String EADAPTER_NAME_TAG = "eis.ResourceAdapterName";
	private static final String EWORKSPACE_TAG = "eis.Workspace"; 
	private static final String EHOSTNAME_TAG = "eis.Hostname"; 
	private static final String EPORT_TAG = "eis.Port";
	private static final String LOGLEVEL_TAG = "Logger.Level";
	
	//For Virtual Adapter.
	private static final String vHOSTNAME_TAG = "va.Hostname";
	private static final String vPort_TAG = "va.Port";
	
	private Properties settings = null;
	private String adapterName = null;
	private String workSpace = null;
	private String hostName = null;
	private int type = -1;
	private int port = -1;
	
	/**
	 * @param path read in properties file path
	 * @param type PropertiesUtil.ADAPTER or PropertiesUtil.EIS
	 */
	public PropertiesUtil(String path, int type){
		this.path = path;
		this.type = type;
		settings = new Properties();
	}
	
	public boolean importProperties(){
		try {
			FileInputStream fileIn = new FileInputStream(path);
			settings.load(fileIn);
		} catch (Exception e) {
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			return false;
		}
		return true;
	}
	
	public String getLoggerLevel(){
		return getProperty(LOGLEVEL_TAG);
	}
	
	public String getAdapterName() {
		return getProperty(EADAPTER_NAME_TAG);
	}

	public String getHostName() {
		if(type == ADAPTER)
			return getProperty(vHOSTNAME_TAG);
		else{
			return getProperty(EHOSTNAME_TAG);
		}
	}

	public int getPort() {
		String value = null;
		if(type == ADAPTER)
			value = getProperty(vPort_TAG);
		else{
			value = getProperty(EPORT_TAG);
		}
		return Integer.parseInt(value);
	}

	
	public String getWorkSpace() {
		return getProperty(EWORKSPACE_TAG);
	}

	private String getProperty(String field){
		return settings.getProperty(field);
	}
	
	public void setAdapterName(String adapterName) {
		this.adapterName = adapterName;
	}

	public void setHostName(String hostName) {
		this.hostName = hostName;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public void setWorkSpace(String workSpace) {
		this.workSpace = workSpace;
	}

	public boolean exportProperties(String location){
		try{
			FileOutputStream fileOut = new FileOutputStream(location);
			settings = new Properties();
			if(type == EIS){
				settings.put(EADAPTER_NAME_TAG, adapterName);
				settings.put(EWORKSPACE_TAG,workSpace);
				settings.put(EHOSTNAME_TAG,hostName);
				settings.put(EPORT_TAG,Integer.toString(port));				
			}else if(type == ADAPTER){
				settings.put(vHOSTNAME_TAG,hostName);
				settings.put(vPort_TAG,Integer.toString(port));
			}
			String title = (type == ADAPTER)?"Virutal Adapter Properties":"EIS Properties";			
			settings.store(fileOut,title);
		}catch(Exception e){			
			EisMockerUtils.getLogger().log(Level.SEVERE, e.getMessage(), e);
			return false;	
		}
		return true;
	}
	
	public String getLogFile(){
		return settings.getProperty("Logger.LogFile");
	}
	
}
