/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.resource.ResourceException;
import javax.resource.cci.Record;
import javax.resource.spi.CommException;
import javax.transaction.xa.Xid;

import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.eismocker.rmidriver.EisEvent;
import com.ibm.eismocker.rmidriver.IDataObjectStore;
import com.ibm.eismocker.rmidriver.RMIObjectClient;
import com.ibm.j2ca.extension.eventmanagement.Event;
import com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid;
import com.ibm.j2ca.extension.eventmanagement.XidImpl;
import com.ibm.j2ca.extension.logging.LogLevel;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.extension.logging.LogUtils;

public class VTAEventStoreWithXid implements EventStoreWithXid {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private RMIObjectClient rimSession = null;

	private IDataObjectStore dataObjectStore = null;

	private LogUtils logUtils;

	public VTAEventStoreWithXid(VTAActivationSpecWithXid actSpec, LogUtils logUtils) throws ResourceException {
		this.logUtils = logUtils;
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid");

		String host = actSpec.getHostname();
		int port = Integer.parseInt(actSpec.getPortnumber());
		// log the properties
		if (logUtils.isTraceEnabled(Level.FINE)) {
			logUtils.trace(Level.FINEST, VTAAdapterConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid", "The host name is " + host);
			logUtils.trace(Level.FINEST, VTAAdapterConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid", "The port is " + port);
		}
		try {
			rimSession = new RMIObjectClient(host, port);
			dataObjectStore = rimSession.getDataObjectStore();
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid", "6801");//$NON-NLS-1$			
			throw new CommException("The connect call has failed. The polling of events from the event store cannot be done.", re);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid");
	}

	public ArrayList getEvents(int arg0, int arg1, String[] arg2) throws ResourceException, CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "getEvents");

		ArrayList events = new ArrayList();
		//CR91998, enable pollquantity
		if (arg0 == 0)
			return events;		
		try {
			ArrayList raEvents = dataObjectStore.getEvents();
			for (int i = 0; i < raEvents.size(); i++) {
				EisEvent metaData = (EisEvent) raEvents.get(i);

				VTAEvent event = new VTAEvent();
				event.setEventId(metaData.getEventId());
				event.setEventType(metaData.getEventType());
				event.setEventStatus(metaData.getStatus());
				event.setOperationName(metaData.getOperation());
				if (metaData.getStatus()==arg1){
					events.add(event);
					//CR91998, enable pollquantity
					--arg0;
					if (arg0 == 0)
						break;					
				}
			}
		} catch (RemoteException re) {
			logUtils.log(LogLevel.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "getEvents", "6802");//$NON-NLS-1$			
			throw new CommException("The polling of events from the event store cannot be done.", re);
		}
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "getEvents");
		return events;
	}

	public Event getEventForXid(XidImpl xid) throws ResourceException, CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "getEventForXid");

		VTAEvent event = new VTAEvent();
		ArrayList raEvents = null;
		boolean found = false;
		try {
			raEvents = dataObjectStore.getEvents();
			for(int idx=0; idx<raEvents.size(); idx++){
				EisEvent metaData= (EisEvent)raEvents.get(idx);
				if(metaData.getXid().equals(xid.toString())){
					event.setEventId(metaData.getEventId());
					event.setEventType(metaData.getEventType());
					event.setEventStatus(metaData.getStatus());
					event.setOperationName(metaData.getOperation());
					found=true;
					break;
				}
			}
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "getEventForXid", "6806", new Object[] {xid.toString()});//$NON-NLS-1$			
			throw new CommException("The corresponding event could not be retrieved for an xid.", re);
		}
		if(!found){
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "getEventForXid", "6806", new Object[] {xid.toString()});//$NON-NLS-1$			
			throw new ResourceException("The corresponding event could not be retrieved for an xid.");			
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "getEventForXid");
		return event;
	}

	public Xid[] getPendingTransactions() throws ResourceException, CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "getPendingTransactions");

		if(dataObjectStore==null){
			throw new ResourceException("The connection to the EIS Mocker is invalid.");			
		}
		ArrayList xids = new ArrayList();
		ArrayList raEvents = null;
		try {
			raEvents = dataObjectStore.getEvents();
		} catch (RemoteException re) {
			logUtils.log(LogLevel.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "getPendingTransactions", "6802");//$NON-NLS-1$
			//throw CommException
			//throw new ResourceException("The polling of events from the event store cannot be done.", e);
			throw new CommException("The polling of events from the event store cannot be done.", re);
		}
		for (int iev = 0; iev < raEvents.size(); iev++) {
			EisEvent metaData = (EisEvent) raEvents.get(iev);
			String xidStr = metaData.getXid();
			if (xidStr != null && !xidStr.equals("") && !xidStr.equals("0")) {
				Xid xid = new XidImpl(xidStr);
				xids.add(xid);
			}
		}
		int size = xids.size();
		Xid xidArry[] = new XidImpl[size];
		for (int ixid  = 0; ixid < size; ixid++) {
			xidArry[ixid] = (Xid) xids.get(ixid);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "getPendingTransactions");
		return xidArry;
	}

	public Record getRecordForEvent(Event event) throws ResourceException, CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "getRecordForEvent");

		VTAStructureRecord record = new VTAStructureRecord();
		try {
			String objectName = event.getEventType();
			String opName = ((VTAEvent) event).getOperationName();
			DataMap dataMap = (DataMap) dataObjectStore.getDataMapForEvent(event.getEventId());

			record.setRecordName(objectName);
			record.setOperationName(opName);
			record.setEISRepresentation(dataMap);
			record.setLogUtils(logUtils);
		}catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "getRecordForEvent", "6805", new Object[] { event.getEventId() });//$NON-NLS-1$			
			throw new CommException("The corresponding data could not be retrieved for an event.", re);			
		}catch (Exception e) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "getRecordForEvent", "6805", new Object[] { event.getEventId() });//$NON-NLS-1$			
			throw new ResourceException("The corresponding data could not be retrieved for an event.", e);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "getRecordForEvent");
		return record;
	}

	public void setEventTransactionID(Event arg0, XidImpl arg1) throws ResourceException, CommException {
	}

	public void deleteEvent(Event event) throws ResourceException, CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "deleteEvent");

		try {
			dataObjectStore.deleteEvent(event.getEventId());
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "deleteEvent", "6804");//$NON-NLS-1$			
			throw new CommException("The adapter could not delete the event.", re);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "deleteEvent");
	}

	public void updateEventStatus(Event event, int newStatus) throws ResourceException, CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAEVENTSTOREXID, "updateEventStatus");
		try {
			dataObjectStore.updateEventStatus(event.getEventId(), newStatus);
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAEVENTSTOREXID, "updateEventStatus", "6803", new Object[] { newStatus+"" });//$NON-NLS-1$			
			throw new CommException("The adapter could not update the event status.", re);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAEVENTSTOREXID, "updateEventStatus");
	}

	public boolean implementsFiltering() {
		return false;
	}

	public boolean isTransactional() {
		return false;
	}

	public void rollbackWork() throws ResourceException, CommException {

	}

	public void commitWork() throws ResourceException, CommException {

	}

	public void close() throws ResourceException, CommException {

	}

}
