/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta;

import java.util.logging.Level;

import javax.resource.ResourceException;
import javax.resource.cci.InteractionSpec;
import javax.resource.cci.Record;
import javax.resource.spi.CommException;

import com.ibm.j2ca.base.DataObjectRecord;
import com.ibm.j2ca.base.WBIConnection;
import com.ibm.j2ca.base.WBIInteraction;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.base.internal.WPSServiceHelper;
import com.ibm.j2ca.extension.commandpattern.CommandForCursor;
import com.ibm.j2ca.extension.commandpattern.CommandManagerForCursor;
import com.ibm.j2ca.extension.dataexchange.bean.DEFactoryJavaBean;
import com.ibm.j2ca.extension.dataexchange.sdo.DEFactorySDO;
import com.ibm.j2ca.extension.emd.runtime.RecordHolderBinding;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.vta.commands.VTABaseCommand;
import com.ibm.j2ca.vta.commands.VTACommandFactory;
import commonj.sdo.DataObject;

public class VTAInteraction extends WBIInteraction {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}
	
	private VTACommandFactory commandFactory = null;
	private CommandManagerForCursor commandMgr = null;
	private VTAInterpreter interpreter = null;
	private VTAManagedConnection mconn = null;
	private LogUtils logUtils = null;


	public VTAInteraction(WBIConnection connection) throws ResourceException{
		super(connection);
		
		mconn = (VTAManagedConnection)connection.getManagedConnection();
		logUtils =  mconn.getLogUtils();
		interpreter = new VTAInterpreter(logUtils);
		commandFactory = new VTACommandFactory();
		commandMgr = new CommandManagerForCursor(commandFactory, mconn.getEISConnection(), logUtils);
		commandMgr.setNamespace("");
	}

	public Record doExecute(InteractionSpec ispec, Record inputRecord)throws ResourceException {
		
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAINTERACTION, "execute");

		Record output = null;
		try {
			WBIInteractionSpec wbiSpec = (WBIInteractionSpec) ispec;
			VTAStructureRecord recordInput = null;
			if(inputRecord instanceof VTAStructureRecord){
				recordInput = (VTAStructureRecord)inputRecord;
			}else if(inputRecord instanceof DataObjectRecord){// is SDO1
				DataObjectRecord record = (DataObjectRecord) inputRecord;
				recordInput = new VTAStructureRecord();
				DEFactorySDO binding = new DEFactorySDO();
				DataObject object = record.getDataObject();
				//Since it would be a BG get the root property
				commonj.sdo.Property prop = WPSServiceHelper.getRootBusinessObjectProperty(object.getType());
				//get the corresponding dataObject from BG
				DataObject inputObject = object.getDataObject(prop);
				binding.setBoundObject(inputObject);
				//initialize the record
				Object[] array = {inputObject};
				recordInput.initializeInput(binding, array);
				recordInput.setNamespace(object.getType().getURI());
				recordInput.setRecordName(object.getType().getName());
				//set the verb as operationName for backward compatability
				//recordInput.setOperationName(object.getString("verb"));
				
				//CR91999 - process the BG case.
				recordInput.setIsBG(true);
				recordInput.setBGVerb(object.getString(VTAAdapterConstants.VERB));
				
			}else{//is JavaBean
				recordInput = new VTAStructureRecord();
				DEFactoryJavaBean binding = new DEFactoryJavaBean();
				binding.setBoundObject(inputRecord);
				Object[] array = {inputRecord};
				recordInput.initializeInput(binding, array);
			}
			recordInput.setLogUtils(logUtils);
			CommandForCursor command = commandMgr.produceCommands(recordInput, wbiSpec.getFunctionName());
			//CR91999 - process the BG case.
			VTABaseCommand vtaCmd = (VTABaseCommand)command;
			vtaCmd.setFuncName(wbiSpec.getFunctionName());
			if(recordInput.getIsBG()){
				vtaCmd.setIsBG(true);
				vtaCmd.setBGVerb(recordInput.getBGVerb());
			}
			
			interpreter.execute(command);
			
			VTAStructureRecord outputRecord = new VTAStructureRecord();
			outputRecord.setEISRepresentation(command.getEisRepresentation());
			outputRecord.setMetadata(recordInput.getMetadata());
			outputRecord.setOperationName(wbiSpec.getFunctionName());
			outputRecord.setLogUtils(logUtils);
			
			output = outputRecord;
			
			if (logUtils.isTraceEnabled(Level.FINE))
				logUtils.traceMethodExit(VTAAdapterConstants.VTAINTERACTION, "execute");
		} catch (CommException ce) {
			throw ce;
		} catch (ResourceException re) {
			throw re;
		} catch (Exception e) {
			logUtils.trace(Level.FINEST, VTAAdapterConstants.VTAINTERACTION, "execute","The execute call on the VTAInteraction instance has failed.", e);
			throw new ResourceException("The execute call on the VTAInteraction instance has failed.", e);
		}		
		return output;
	}

	public boolean doExecute(InteractionSpec arg0, Record inRecord, Record outRecord) throws ResourceException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAINTERACTION, "execute");

		boolean returnValue = false;
		try{
			VTAStructureRecord outputRecord = (VTAStructureRecord)this.execute(arg0, inRecord);
			returnValue = true;

			//we need to process the output for SDO1,SDO2,JavaBean. convert it to the correct data type.
			if(outRecord instanceof VTAStructureRecord){
				outRecord = outputRecord;
			}else if(outRecord instanceof DataObjectRecord){// is SDO1
				DataObject bg = null;
				DEFactorySDO dataBinding = new DEFactorySDO();
				DataObject dataObject = WPSServiceHelper.createBusinessObject(outputRecord.getNamespace(), outputRecord.getRecordName());
				Object[] array = {dataObject.getType()};
				outputRecord.initializeOutput(dataBinding, array);
				
				if(outputRecord.getOperationName().equalsIgnoreCase(WBIInteractionSpec.RETRIEVE_ALL_OP)){//build up the container.
					DataObject returnObject = WPSServiceHelper.createBusinessObject(dataObject.getType().getURI() + "container", dataObject.getType().getName() + "Container");
					while (outputRecord.getNext(true)) {
						DataObject BG = WPSServiceHelper.createDataObject(outputRecord.getNamespace(), outputRecord.getRecordName());
						BG.set(WPSServiceHelper.getRootBusinessObjectProperty(BG.getType()), dataBinding.getBoundObject());
						returnObject.getList(0).add(BG);
					}
					bg = returnObject;
				}else{//is create/update/delete/exist/retrieve operation.
					outputRecord.getNext(true);
					bg = WPSServiceHelper.getBusinessGraph(dataObject);
					if (bg != null) {//if BG is present
						bg.set(WPSServiceHelper.getRootBusinessObjectProperty(bg.getType()), dataBinding.getBoundObject());
						if (outputRecord.getOperationName().equalsIgnoreCase(WBIInteractionSpec.CREATE_OP) 
								|| outputRecord.getOperationName().equalsIgnoreCase(WBIInteractionSpec.UPDATE_OP)
								|| outputRecord.getOperationName().equalsIgnoreCase(WBIInteractionSpec.DELETE_OP)){
							bg.set(VTAAdapterConstants.VERB, outputRecord.getOperationName());
						}
					}else {//set the returned BO
						bg = (DataObject) dataBinding.getBoundObject();
					}
				}
			}else{//Java Bean
				((RecordHolderBinding) outRecord).setRecord(outputRecord);
			}

		}  catch (CommException ce) {
			throw ce;
		}catch (Exception e) {
			logUtils.trace(Level.FINEST, VTAAdapterConstants.VTAINTERACTION, "execute","The execute call on the VTAInteraction instance has failed.", e);
			throw new ResourceException("The execute call on the VTAInteraction instance has failed.", e);
		}
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAINTERACTION, "execute");
		return returnValue;
	}
	
	
	
}
