/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;

import javax.resource.ResourceException;

import com.ibm.despi.InputAccessor;
import com.ibm.despi.InputCursor;
import com.ibm.despi.OutputAccessor;
import com.ibm.despi.OutputCursor;
import com.ibm.despi.exception.DESPIException;
import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.extension.metadata.Property;
import com.ibm.j2ca.extension.metadata.Type;
import com.ibm.j2ca.vta.commands.VTABaseCommand;

public class VTAUtil {
	static String copyright() {
		return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	public static DataMap convertRecordToDataMap(InputCursor inputObject, Type metadata, VTABaseCommand parentCommand, LogUtils logUtils) throws DESPIException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAUTIL, "convertRecordToDataMap");

		String boName = metadata.getName();
		DataMap outDataMap = new DataMap(boName);
		try {
			copyRecordToDataMap(inputObject, metadata, outDataMap,logUtils);
		} catch (ResourceException re) {
			throw new DESPIException(re.getMessage());
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAUTIL, "convertRecordToDataMap");
		return outDataMap;
	}

	public static void copyDataMapToRecord(DataMap inDataMap, OutputCursor outputObject, Type metadata, LogUtils logUtils) throws DESPIException {

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAUTIL, "copyDataMapToRecord");

		for (Iterator e = metadata.getPropertyIterator(); e.hasNext();) {
			Property prop = (Property) e.next();
			String propName = prop.getName();
			Object value = inDataMap.get(propName);
			if (value == null) {
				continue;
			}
			if (prop.isContainment()) {// is Child BO
				if (prop.isMany()) {// N-Card
					Type childMetadata = prop.getType();
					List childList = inDataMap.getList(propName);
					OutputCursor childCursor = (OutputCursor) outputObject.getChildCursor(propName);
					for (int j = 0; j < childList.size(); j++) {
						DataMap childDataMap = (DataMap) childList.get(j);
						childCursor.startObject();
						copyDataMapToRecord(childDataMap, childCursor, childMetadata,logUtils);
						childCursor.completeObject();
					}
				} else {// 1-Card
					Type childMetadata = prop.getType();
					OutputCursor childCursor = (OutputCursor) outputObject.getChildCursor(propName);
					DataMap childDataMap = (DataMap) value;
					childCursor.startObject();
					copyDataMapToRecord(childDataMap, childCursor, childMetadata,logUtils);
					childCursor.completeObject();
				}
			} else {// is Simple type
				((OutputAccessor) outputObject.getAccessor(propName)).setObject(value);
			}
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAUTIL, "copyDataMapToRecord");
	}

	public static void copyRecordToDataMap(InputCursor inputObject, Type metadata, DataMap outDataMap, LogUtils logUtils) throws DESPIException, ResourceException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTAUTIL, "copyDataMapToRecord");
	
		for (Iterator e = metadata.getPropertyIterator(); e.hasNext();) {
			Property prop = (Property) e.next();
			String propName = prop.getName();
			if (prop.isContainment()) {// is Child BO
					InputCursor childCursor = (InputCursor)inputObject.getChildCursor(propName);
					childCursor.reset();
					while(childCursor.getNext()){
						DataMap childDataMap = outDataMap.createDataMap(propName,prop.isMany());
						copyRecordToDataMap(childCursor, prop.getType(),childDataMap,logUtils);						
					}
			} else {// is simple type
				InputAccessor accessor = (InputAccessor) inputObject.getAccessor(propName);
				if (accessor != null && accessor.isSet()) {
					if(prop.isMany()){//N-Card
						List valueList = (List)accessor.getObject();
						ArrayList newList = new ArrayList();
						for(int idx=0;idx<valueList.size();idx++){
							newList.add(valueList.get(idx));
						}
						outDataMap.set(propName, newList);
					}else{//1-Card
						outDataMap.set(propName, accessor.getObject());
					}
				}
			}
		}
		
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTAUTIL, "copyDataMapToRecord");
	}

}
