/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.commands;

import java.rmi.RemoteException;
import java.util.logging.Level;

import javax.resource.ResourceException;
import javax.resource.spi.CommException;

import com.ibm.despi.InputCursor;
import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.eismocker.rmidriver.IDataObjectStore;
import com.ibm.eismocker.rmidriver.RMIObjectClient;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.extension.commandpattern.CommandForCursor;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.extension.metadata.Type;
import com.ibm.j2ca.vta.VTAAdapterConstants;
import com.ibm.j2ca.vta.VTAUtil;

public class VTABaseCommand extends CommandForCursor {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}
	private RMIObjectClient client = null;
	private IDataObjectStore dataObjStore = null;
	private String operationName =null;
	private LogUtils logUtils =null;
	
	private String funcName="";
	private String bgVerb="";
	private boolean isBG=false;

	public VTABaseCommand(String nodeLevelOperation){
		operationName = nodeLevelOperation;
		logUtils = getLogUtils();
	}
	public void execute(InputCursor inputObject, Type metadata)throws ResourceException {
		logUtils = getLogUtils();
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAAdapterConstants.VTABASECOMMAND, "execute");

		try{
			client = (RMIObjectClient)this.getConnection();
			dataObjStore = client.getDataObjectStore();
		}catch(RemoteException re){
			logUtils.trace(Level.FINEST, VTAAdapterConstants.VTABASECOMMAND, "execute","The Adapter failed to establish session with EIS Mocker", re);			
			throw new CommException("The Adapter failed to establish session with EIS Mocker",re);			
		}
		try {
			//CR91999 - process the BG case.
			//DataMap inputDataMap = VTAUtil.convertRecordToDataMap(inputObject, metadata,this,this.getLogUtils());
			//DataMap outputDataMap = dataObjStore.sendDataMap(inputDataMap,operationName);
			DataMap inputDataMap = null;
			if(this.getIsBG()){
				String boName = metadata.getName();
				String bgName = boName + "BG";
				inputDataMap = new DataMap(bgName);
				DataMap boDataMap = inputDataMap.createDataMap(boName, false);
				inputDataMap.set(VTAAdapterConstants.VERB, this.bgVerb);
				VTAUtil.copyRecordToDataMap(inputObject, metadata, boDataMap,logUtils);
			}else{
				inputDataMap = VTAUtil.convertRecordToDataMap(inputObject, metadata,this,logUtils);
			}
			DataMap outputDataMap = dataObjStore.sendDataMap(inputDataMap,this.getFuncName());
			//CR91999 End
			this.setEisRepresentation(outputDataMap);
			if(WBIInteractionSpec.EXISTS_OP.equalsIgnoreCase(operationName)){
				this.setEisRepresentation(new Boolean(true));				
			}
		}catch (RemoteException re) {
			logUtils.trace(Level.FINEST, VTAAdapterConstants.VTABASECOMMAND, "execute","The Adapter failed to send data to EIS Mocker", re);			
			throw new CommException("The Adapter failed to send data to EIS Mocker",re);
		}catch (Exception e) {
			throw new ResourceException("The execute call on the VTABaseCommand instance has failed",e);
		}
		
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAAdapterConstants.VTABASECOMMAND, "execute");
	}
	//CR91999 - process the BG case.
	public void setBGVerb(String verb){
		this.bgVerb = verb;
	}
	public String getBGVerb(){
		return this.bgVerb;
	}
	public void setFuncName(String op){
		this.funcName = op;
	}
	public String getFuncName(){
		return this.funcName;
	}
	public void setIsBG(boolean bg){
		this.isBG = bg;
	}
	public boolean getIsBG(){
		return this.isBG;
	}
	//End
}
