/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.description;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.logging.Level;

import javax.xml.namespace.QName;

import com.ibm.j2ca.base.TopLevelVerbs;
import com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIMetadata;
import com.ibm.j2ca.vta.emd.discovery.VTAAttributeType;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataImportConfiguration;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataObject;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.SchemaDefinition;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataObjectResponse;

public class VTADataDescription extends WBIDataDescriptionImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private StringBuffer includeXSDs = new StringBuffer();

	private MetadataImportConfiguration[] selectedSet;

	private StringBuffer includeXSDsSecondary = new StringBuffer();

	public VTADataDescription(VTAMetadataObject object, MetadataImportConfiguration[] selectedSet) throws MetadataException {
		super.setMetadataObject(object);
		super.setAliasTag(object.getAliasTag());
		this.selectedSet = selectedSet;
	}

	public void populateSchemaDefinitions() throws MetadataException {
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodEntrance("VTADataDescription", "populateSchemaDefinitions");

		try {
			prepareChildSchemaFiles();
			VTAMetadataObject object = (VTAMetadataObject) getMetadataObject();
			// Extract the content after the tag for ComplexType> This is needed
			// to add the annotation for object level ASI
			String complexTypeInfo = object.getContent().substring(0, object.getContent().indexOf(">") + 1) + System.getProperty("line.separator");
			String content;
			String header = object.getHeader();
			int encoding = header.indexOf("encoding");
			if (encoding != -1) {
				String encodingName = header.substring(encoding + 10, header.indexOf("\"", encoding + 10));
				if (!encodingName.equalsIgnoreCase("UTF-8")) {
					header = header.substring(0, encoding + 10) + "UTF-8" + header.substring(header.indexOf("\"", encoding + 10));
				}
			}
			content = header + includeXSDs.toString() + getSerializedImports(this.getImportNameSpaces()) + System.getProperty("line.separator") + getSerializedAnnotationForEMD(getASISchemaName())
					+ complexTypeInfo + getSerializedMetadata(this.getMetadataForBusinessObject()) + object.getContent().substring(object.getContent().indexOf(">") + 1);
			// Add defn to schema defintion list
			URI uri = new URI(this.getRelativePath() + getBOName() + ".xsd");
			put(object.getTargetNameSpace(), uri, content);
		} catch (Exception e) {
			throw new MetadataException("Unable to create Data Description from Object selected", e);
		}
		
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodExit("VTADataDescription", "populateSchemaDefinitions");
	}

	public String getASISchemaName() {
		return null;
	}

	public String getAttributeName(String attrName) {
		return attrName;
	}

	public String getCardinality(String name) {
		LinkedHashMap ht = this.getAttributeList();
		VTAAttributeType vtaAttrType = (VTAAttributeType) ht.get(name);
		return vtaAttrType.getCardinality();
	}

	public Iterator getChildList() throws MetadataException {
		return (this.getMetadataObject().getChildren(null)).getObjectIterator();
	}

	public List getImportNameSpaces() throws MetadataException {
		ArrayList list = new ArrayList();
		return list;
	}

	public int getMaxLength(String name) {
		return 0;
	}

	public WBIMetadata getMetadataForAttribute(String attrName) {
		return null;
	}

	public WBIMetadata getMetadataForBusinessObject() {
		return null;
	}

	public List getNameSpaces() {
		ArrayList list = new ArrayList();
		list.add(new QName("http://www.ibm.com/xmlns/prod/websphere/j2ca/vta", "pasi"));
		return list;
	}

	public boolean getRequired(String name) {
		return false;
	}

	public String getType(String attrName) {
		LinkedHashMap ht = this.getAttributeList();
		VTAAttributeType vtaAttrType = (VTAAttributeType) ht.get(attrName);
		return vtaAttrType.getType();
	}

	public List getVerbs() {
		ArrayList list = new ArrayList();
		list.add(TopLevelVerbs.CREATE_TLV);
		list.add(TopLevelVerbs.UPDATE_TLV);
		list.add(TopLevelVerbs.DELETE_TLV);
		return list;
	}

	public boolean isContainer() {
		boolean retValue = false;
		String objName = getName().getLocalPart();
		if (objName.endsWith("Container") && !objName.equals(getBOName()))
			retValue = true;
		return retValue;
	}

	public void prepareChildSchemaFiles() throws MetadataException {
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodEntrance("VTADataDescription", "prepareChildSchemaFiles");

		MetadataObjectResponse response = getMetadataObject().getChildren(null);
		VTAMetadataObject child = null;
		Iterator i = response.getObjectIterator();
		while (i.hasNext()) {
			child = (VTAMetadataObject) i.next();
			if (child.getBOType() != null) {
				if (child.getTargetNameSpace().equals(((VTAMetadataObject) this.getMetadataObject()).getTargetNameSpace()) && child.isPartOfSameXSD()) {
					includeXSDs.append("<" + child.getAliasTag() + "include schemaLocation=\"" + child.getBOName() + ".xsd\" />\n");
				}

				boolean found = false;
				for (int j = 0; j < selectedSet.length; j++) {
					VTAMetadataObject selectedObj = (VTAMetadataObject) ((VTAMetadataImportConfiguration) selectedSet[j]).getMetadataObject();
					if (selectedObj.getLocation().equals(child.getLocation())) {
						found = true;
						break;
					}
				}
				if (found)
					continue;
				// Generate the schemas for grand and following level children
				VTADataDescription desc = new VTADataDescription(child, selectedSet);
				desc.setRelativePath(this.getRelativePath());
				desc.setAliasTag(child.getAliasTag());
				desc.prepareChildSchemaFiles();
				SchemaDefinition[] schemas = desc.getSchemaDefinitions();
				for (int j = 0; j < schemas.length; j++) {
					SchemaDefinition defn = schemas[j];
					put(defn.getNamespace(), defn.getLocation(), defn.getContent());
				}
				// If not found continue to add the defn of the child object
				String complexTypeInfo = child.getContent().substring(0, child.getContent().indexOf(">") + 1) + System.getProperty("line.separator");
				String header = child.getHeader();
				int encoding = header.indexOf("encoding");
				if (encoding != -1) {
					String encodingName = header.substring(encoding + 10, header.indexOf("\"", encoding + 10));
					if (!encodingName.equalsIgnoreCase("UTF-8"))
						header = header.substring(0, encoding + 10) + "UTF-8" + header.substring(header.indexOf("\"", encoding + 10));
				}
				String content = header + desc.getIncludeXSDs() + getSerializedImports(this.getImportNameSpaces()) + System.getProperty("line.separator")
						+ getSerializedAnnotationForEMD(getASISchemaName()) + complexTypeInfo + getSerializedMetadata(this.getMetadataForBusinessObject())
						+ child.getContent().substring(child.getContent().indexOf(">") + 1);
				URI uri = null;
				try {
					uri = new URI(this.getRelativePath() + child.getBOName() + ".xsd");
				} catch (URISyntaxException use) {
					throw new MetadataException(use.getMessage(),use);
				}
				
				put(child.getTargetNameSpace(), uri, content);

			} else {

				VTADataDescription boDesc = new VTADataDescription(child, selectedSet);
				boDesc.setRelativePath(this.getRelativePath());
				boDesc.setAliasTag(child.getAliasTag());
				boDesc.populateSchemaDefinitions();
				SchemaDefinition[] schemas = boDesc.getSchemaDefinitions();
				for (int j = 0; j < schemas.length; j++) {
					SchemaDefinition defn = schemas[j];
					put(defn.getNamespace(), defn.getLocation(), defn.getContent());
				}
			}
		}

		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodExit("VTADataDescription", "prepareChildSchemaFiles");
	}

	public boolean isTopLevel() {
		return false;
	}

	protected String getIncludeXSDs() {
		return this.includeXSDs.toString();
	}

	protected String getIncludeXSDsSecondary() { // cr84766
		return this.includeXSDsSecondary.toString();
	}

}
