/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.description;

import java.util.ArrayList;
import java.util.logging.Level;

import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.description.WBIInboundFunctionDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIInboundServiceDescriptionImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataImportConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIMultiValuedPropertyImpl;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataObject;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.FunctionDescription;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataSelection;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAInboundServiceDescription extends WBIInboundServiceDescriptionImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	PropertyNameHelper helper = null;
	
	public VTAInboundServiceDescription(PropertyNameHelper helper){
		super(helper);
		this.helper = helper;
	}
	
	public void setFunctionDescriptions(MetadataSelection selection) throws MetadataException {
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodEntrance("VTAInboundServiceDescription", "setFunctionDescriptions");

		try{
		ArrayList functionDescriptions = new ArrayList();
		MetadataImportConfiguration[] confArray = selection.getSelection();
		PropertyGroup selectionProps = WBIMetadataSelectionImpl.getAppliedSelectionProperties();
		WBIMultiValuedPropertyImpl prop = (WBIMultiValuedPropertyImpl) selectionProps.getProperty("Operations");
		String[] values = prop.getValuesAsStrings();
		
		helper.getToolContext().getProgressMonitor().setMaximum(confArray.length * values.length * 10);
		helper.getToolContext().getProgressMonitor().setMinimum(0);
		helper.getToolContext().getProgressMonitor().setNote("Creating Service Description");
		
		for (int i = 0; i < confArray.length; i++) {
			if (helper.getToolContext().getProgressMonitor().isCanceled()) {
				return;
			}

			WBIMetadataImportConfigurationImpl spec = (WBIMetadataImportConfigurationImpl) confArray[i];
			WBIInboundFunctionDescriptionImpl funcDesc;
			VTAMetadataObject metadataObj = (VTAMetadataObject) spec.getMetadataObject();
			helper.getToolContext().getProgressMonitor().setNote("Creating Service for " + metadataObj.getDisplayName());
			// xingcl
			this.setNameSpace(metadataObj.getTargetNameSpace());
			VTADataDescription dataDesc = VTAOutboundServiceDescription.setupMetadataObject(spec, selection, getNameSpace());
			for (int j = 0; j < values.length; j++) {
				String operation = values[j];
				helper.getToolContext().getProgressMonitor().setProgress(i + j * 10);
				funcDesc = new WBIInboundFunctionDescriptionImpl();
				char firstCharacter = operation.charAt(0);
				funcDesc.setName("emit" + Character.toUpperCase(firstCharacter) + operation.substring(1).toLowerCase() + "AfterImage" + metadataObj.getBOName());
				funcDesc.setEISFunctionName("emit" + Character.toUpperCase(firstCharacter) + operation.substring(1).toLowerCase() + "AfterImage" + dataDesc.getName().getLocalPart());
				funcDesc.setImportConfiguration(spec);
				funcDesc.setInputDataDescription(dataDesc);
				functionDescriptions.add(funcDesc);
			}
		}
		FunctionDescription[] funcArray = new FunctionDescription[functionDescriptions.size()];
		functionDescriptions.toArray(funcArray);
		super.setFunctionDescriptions(funcArray);
		
		}catch(Exception e){
			super.getLogUtils().trace(Level.FINE, "VTAInboundServiceDescription", "setFunctionDescriptions", "",e);
			throw new MetadataException(e.getMessage(),e);
		}

		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodExit("VTAInboundServiceDescription", "setFunctionDescriptions");

	}
}
