/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.description;

import java.util.ArrayList;
import java.util.logging.Level;

import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.description.WBIOutboundFunctionDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIOutboundServiceDescriptionImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataImportConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIMultiValuedPropertyImpl;
import com.ibm.j2ca.vta.VTAInteractionSpec;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataObject;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataSelection;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.FunctionDescription;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataSelection;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAOutboundServiceDescription extends WBIOutboundServiceDescriptionImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	PropertyNameHelper helper = null;

	public VTAOutboundServiceDescription(PropertyNameHelper helper) {
		super(helper);
		this.helper = helper;
	}

	public void setFunctionDescriptions(MetadataSelection selection) throws MetadataException {
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodEntrance("VTAOutboundServiceDescription", "setFunctionDescriptions");

		ArrayList functionDescriptions = new ArrayList();
		MetadataImportConfiguration[] confArray = selection.getSelection();
		PropertyGroup selectionProps = WBIMetadataSelectionImpl.getAppliedSelectionProperties();
		WBIMultiValuedPropertyImpl prop = (WBIMultiValuedPropertyImpl) selectionProps.getProperty("Operations"); //$NON-NLS-1$
		String[] values = prop.getValuesAsStrings();

		helper.getToolContext().getProgressMonitor().setMaximum(confArray.length * values.length * 10);
		helper.getToolContext().getProgressMonitor().setMinimum(0);
		helper.getToolContext().getProgressMonitor().setNote("Creating Service Description");

		for (int i = 0; i < confArray.length; i++) {
			WBIMetadataImportConfigurationImpl spec = (WBIMetadataImportConfigurationImpl) confArray[i];
			WBIOutboundFunctionDescriptionImpl funcDesc;
			VTAInteractionSpec iSpec;
			VTAMetadataObject metadataObj = (VTAMetadataObject) spec.getMetadataObject();
			helper.getToolContext().getProgressMonitor().setNote("Creating Service for " + metadataObj.getDisplayName());

			this.setNameSpace(metadataObj.getTargetNameSpace());
			VTADataDescription dataDesc = setupMetadataObject(spec, selection, getNameSpace());
			for (int j = 0; j < values.length; j++) {
				String operation = (String) values[j];
				//CR92038 - "ApplyChanges" operation is only applicable for BG. 
				if(operation.equalsIgnoreCase(WBIInteractionSpec.APPLY_CHANGES_OP) && (!metadataObj.getBOName().endsWith("BG"))){
					continue;
				}
				helper.getToolContext().getProgressMonitor().setProgress(i + j * 10);
				funcDesc = new WBIOutboundFunctionDescriptionImpl();
				funcDesc.setName(operation.toLowerCase() + metadataObj.getBOName());
				funcDesc.setInputDataDescription(dataDesc);
				funcDesc.setOutputDataDescription(dataDesc);
				iSpec = new VTAInteractionSpec();
				iSpec.setFunctionName(operation);
				funcDesc.setInteractionSpec(iSpec);
				funcDesc.setImportConfiguration(spec);
				functionDescriptions.add(funcDesc);
			}
		}

		FunctionDescription[] funcArray = new FunctionDescription[functionDescriptions.size()];
		functionDescriptions.toArray(funcArray);
		super.setFunctionDescriptions(funcArray);

		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodExit("VTAOutboundServiceDescription", "setFunctionDescriptions");

	}

	public static VTADataDescription setupMetadataObject(WBIMetadataImportConfigurationImpl spec, MetadataSelection selection, String namespace) throws MetadataException {
		VTAMetadataObject metadataObj = (VTAMetadataObject) spec.getMetadataObject();

		String location = "./";
		VTADataDescription dataDesc = new VTADataDescription(metadataObj, selection.getSelection());

		dataDesc.setMetadataObject(metadataObj);
		dataDesc.setName(namespace, metadataObj.getDisplayName());
		dataDesc.setRelativePath(location);

		String artifactType = ((VTAMetadataSelection) selection).getSelectedArtifactType();
		if (artifactType.equalsIgnoreCase(MetadataConfigurationType.GENERATED_DATA_BINDING.toString())) {// WPS
			dataDesc.setTopLevel(false);
			dataDesc.populateSchemaDefinitions();
			dataDesc.setDataBindingGeneratorClassName("com.ibm.j2ca.vta.emd.runtime.VTADataBindingGenerator");
			dataDesc.setGenericDataBindingClassName(null);
		} else if (artifactType.equalsIgnoreCase(MetadataConfigurationType.GENERATED_RECORDS.toString())) {// WAS
			dataDesc.setTopLevel(false);
			dataDesc.populateSchemaDefinitions();
			dataDesc.setDataBindingGeneratorClassName("com.ibm.j2ca.extension.dataexchange.bean.generator.RecordGenerator");
			dataDesc.setGenericDataBindingClassName(null);

		} else {// WMB
			dataDesc.setTopLevel(false);
			dataDesc.populateSchemaDefinitions();
			dataDesc.setGenericDataBindingClassName("com.ibm.j2ca.vta.VTAStructureRecord");
		}
		return dataDesc;
	}

}
