/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.discovery;

import java.util.logging.Level;
import java.util.logging.LogRecord;

import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.WBIAdapterTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIMetadataConnectionImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionTypeImpl;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.vta.emd.description.VTAInboundServiceDescription;
import com.ibm.j2ca.vta.emd.description.VTAOutboundServiceDescription;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAInboundConnectionType;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAOutboundConnectionConfiguration;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAOutboundConnectionType;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.ServiceDescription;
import commonj.connector.metadata.discovery.AdapterType;
import commonj.connector.metadata.discovery.AdapterTypeSummary;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataSelection;
import commonj.connector.metadata.discovery.MetadataTree;
import commonj.connector.metadata.discovery.connection.MetadataConnection;
import commonj.connector.tool.ToolContext;

public class VTAMetadataDiscovery extends WBIMetadataDiscoveryImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	WBIAdapterTypeImpl adapterType = null;

	WBIMetadataConnectionImpl connection;

	private static WBIOutboundConnectionTypeImpl connType;

	PropertyNameHelper helper = null;

	public VTAMetadataDiscovery() throws MetadataException {
		super("com.ibm.j2ca.vta.emd");
		helper = new PropertyNameHelper(null, "com.ibm.j2ca.vta.emd", null);
	}

	public ServiceDescription createServiceDescription(MetadataSelection importSelection) throws MetadataException {

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataDiscovery", "createServiceDescription");

		ServiceDescription description = null;
		MetadataImportConfiguration[] confArray = importSelection.getSelection();
		if (confArray.length == 0)
			return description;

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Number of MetadataImportConfigurations " + confArray.length);

		String serviceType = ((VTAMetadataSelection) importSelection).getSelectedServiceType();

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Selected Service Type is " + serviceType);

		boolean inbound = serviceType.equalsIgnoreCase(MetadataConfigurationType.INBOUND_SERVICE.toString());

		//PropertyGroup pg = WBIMetadataSelectionImpl.getAppliedSelectionProperties();
		//SingleValuedProperty prop = (SingleValuedProperty) pg.getProperty("NameSpace");
		//String namespaceName = (String) prop.getValue();

		if (inbound) {
			VTAInboundServiceDescription iDesc = new VTAInboundServiceDescription(helper);
			String serviceName = "VTAInboundInterface"; //$NON-NLS-1$ 
			iDesc.setName(serviceName);
			// iDesc.setNameSpace(namespaceName);

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Setting function Descriptions");

			iDesc.setFunctionDescriptions(importSelection);
			WBIInboundConnectionConfigurationImpl inboundConf = (WBIInboundConnectionConfigurationImpl) adapterType.getInboundConnectionTypes()[0].createInboundConnectionConfiguration();
			// Set the BONameSpace as the one that is preselected
			inboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
			iDesc.setInboundConnectionAdvancedConfiguration(inboundConf);
			iDesc.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
			iDesc.setComment("Service Description for Inbound");
			iDesc.setMetadataSelectionProperties(WBIMetadataSelectionImpl.getAppliedSelectionProperties());
			description = iDesc;

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Inbound Service Description Created Successfully");

		} else {
			VTAOutboundServiceDescription oDesc = new VTAOutboundServiceDescription(helper);
			String serviceName = "VTAOutboundInterface";
			oDesc.setName(serviceName);
			// oDesc.setNameSpace(namespaceName);

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Setting function Descriptions");

			oDesc.setFunctionDescriptions(importSelection);
			oDesc.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
			oDesc.setComment("Service Description for Outbound");
			oDesc.setMetadataSelectionProperties(WBIMetadataSelectionImpl.getAppliedSelectionProperties());
			VTAOutboundConnectionConfiguration outboundConf = (VTAOutboundConnectionConfiguration) adapterType.getOutboundConnectionTypes()[1].createOutboundConnectionConfiguration();
			outboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
			oDesc.setOutboundConnectionAdvancedConfiguration(outboundConf);
			description = oDesc;

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Oubound Service Description Created Successfully");
		}

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataDiscovery", "createServiceDescription");

		return description;
	}

	public MetadataTree getMetadataTree(MetadataConnection conn) throws MetadataException {
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataDiscovery", "getMetadataTree");

		this.connection = (WBIMetadataConnectionImpl) conn;
		connType = (WBIOutboundConnectionTypeImpl) connection.getConnectionType();

		VTAMetadataTree tree = new VTAMetadataTree(connection, helper);
		tree.setSelectionStyle(MetadataTree.MULTI_SELECT);

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataDiscovery", "getMetadataTree");

		return tree;
	}

	public AdapterType getAdapterType(String adapterTypeID) throws MetadataException {
		if (adapterType == null)
			this.getAdapterTypeSummaries();
		return adapterType;
	}

	public AdapterTypeSummary[] getAdapterTypeSummaries() throws MetadataException {

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataDiscovery", "getAdapterTypeSummaries");

		adapterType = new WBIAdapterTypeImpl("com.ibm.j2ca.vta.VTAResourceAdapter", 2, 1);
		adapterType.setDescription("VTA");
		adapterType.setDisplayName("VTA");
		adapterType.setId("VTA");
		adapterType.setVendor("IBM");
		adapterType.setVersion("6.1");

		VTAOutboundConnectionType outConnTypeForMetadata = new VTAOutboundConnectionType(adapterType, helper);
		outConnTypeForMetadata.setIsSupportedInMetadataService(true);
		outConnTypeForMetadata.setIsSupportedAtRuntime(false);
		outConnTypeForMetadata.setManagedConnectionFactoryJavaBean("com.ibm.j2ca.vta.VTAManagedConnectionFactory"); //$NON-NLS-1$
		outConnTypeForMetadata.setResourceAdapterJavaBean("com.ibm.j2ca.vta.VTAResourceAdapter"); //$NON-NLS-1$
		adapterType.addOutboundConnectionType(outConnTypeForMetadata);

		VTAOutboundConnectionType outConnTypeForRuntime = new VTAOutboundConnectionType(adapterType, helper);
		outConnTypeForRuntime.setIsSupportedInMetadataService(false);
		outConnTypeForRuntime.setIsSupportedAtRuntime(true);
		outConnTypeForRuntime.setManagedConnectionFactoryJavaBean("com.ibm.j2ca.vta.VTAManagedConnectionFactory"); //$NON-NLS-1$
		outConnTypeForRuntime.setResourceAdapterJavaBean("com.ibm.j2ca.vta.VTAResourceAdapter"); //$NON-NLS-1$
		adapterType.addOutboundConnectionType(outConnTypeForRuntime);

		VTAInboundConnectionType inConnTypeForRuntime = new VTAInboundConnectionType(adapterType, helper);
		inConnTypeForRuntime.setResourceAdapterJavaBean("com.ibm.j2ca.vta.VTAResourceAdapter"); //$NON-NLS-1$
		inConnTypeForRuntime.setActivationSpecJavaBean("com.ibm.j2ca.vta.VTAActivationSpecWithXid"); //$NON-NLS-1$
		adapterType.addInboundConnectionType(inConnTypeForRuntime);

		return new WBIAdapterTypeImpl[] { adapterType };
	}

	public MetadataConfigurationType[] getSupportedConfiguration() {
		MetadataConfigurationType[] array = new MetadataConfigurationType[5];
		array[0] = MetadataConfigurationType.GENERATED_DATA_BINDING;
		array[1] = MetadataConfigurationType.GENERIC_RECORDS;
		array[2] = MetadataConfigurationType.GENERATED_RECORDS;
		array[3] = MetadataConfigurationType.INBOUND_SERVICE;
		array[4] = MetadataConfigurationType.OUTBOUND_SERVICE;
		return array;
	}

	public void setToolContext(ToolContext toolContext) {
		super.setToolContext(toolContext);
		try {
			AdapterType adapterType = new WBIAdapterTypeImpl("com.ibm.j2ca.vta.VTAResourceAdapter", 2, 1);
			LogUtils logUtils = new LogUtils(toolContext.getLogger(), "com.ibm.j2ca.vta.emd", adapterType.getId(), adapterType.getVersion());
			LogRecord record = new LogRecord(Level.FINEST, "Entering setToolContext"); //$NON-NLS-1$ 
			toolContext.getLogger().log(record);
			helper.setLogUtils(logUtils);
			helper.setToolContext(toolContext);
			super.setLogUtils(logUtils);
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}

	}

}
