/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.discovery;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;

import com.ibm.j2ca.base.xsdutil.BOType;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataObjectImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataObjectResponseImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataObjectResponse;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAMetadataObject extends WBIMetadataObjectImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private VTAMetadataTree tree;
	private ArrayList childList;
	String content;
	String targetNameSpace;
	String header;
	String charSet;
	BOType boType;
	String aliasTag;
	String xsdName;
	boolean containedChild;
	boolean partOfSameXSD;

	PropertyNameHelper helper=null;
	
	public VTAMetadataObject(VTAMetadataTree tree,PropertyNameHelper helper) {
		this.tree = tree;
		this.helper = helper;
	}

	public PropertyGroup createFilteringProperties() {
		return null;
	}

	public MetadataObjectResponse getChildren(PropertyGroup filterParameters) throws MetadataException {
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataObject", "getChildren");
		
		WBIMetadataObjectResponseImpl response = new WBIMetadataObjectResponseImpl();
		ArrayList list = new ArrayList();
		try {
			if (boType != null) {
				List children = boType.getChildren();
				if (children != null)
					for (Iterator e = children.iterator(); e.hasNext();) {
						BOType child = (BOType) e.next();
						String name = child.getName();
						// Create the child MetadataObject
						VTAMetadataObject childObject = new VTAMetadataObject(tree,helper);
						childObject.setTargetNameSpace(child.getType().getNameSpace());
						childObject.setDisplayName(name);
						childObject.setBOType(child);
						if (child.getChildren() != null && child.getChildren().size() > 0) {
							childObject.setHasChildren(true);
						}
						// Mark these as not selectable
						childObject.setSelectableForImport(false);
						childObject.setContainedChild(true);
						//childObject.setContentType(getContentType());
						childObject.setBOName(name);
						
						// Set the parent object
						childObject.setParent(this);
						childObject.setLocation(childObject.getTargetNameSpace() + childObject.getBOName());
						
						// Copy the content and header from the
						// object created for the same complexType
						// as a top level object. This would have happened
						// in the step of DTFMetadataTree.listMetadataObjects()
						VTAMetadataObject topObject = (VTAMetadataObject) tree.getMetadataObject(childObject.getLocation());
						childObject.setHeader(topObject.getHeader());
						childObject.setAliasTag(topObject.getAliasTag());
						childObject.setContent(topObject.getContent());
						childObject.setXsdName(topObject.getXsdName());
						if (childObject.getXsdName().equals(this.getXsdName())) {
							childObject.setPartOfSameXSD(true);
							topObject.setPartOfSameXSD(true);
						}
						list.add(childObject);
					}
			} else {
				if (childList != null)
					list = childList;
			}
		} catch (Exception e) {
			throw new MetadataException(e.getMessage());
		}
		response.setObjects(list);

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataObject", "getChildren");

		return response;
	}

	public PropertyGroup getObjectProperties() {
		WBIPropertyGroupImpl pg = null;
		return pg;
	}

	public MetadataImportConfiguration createImportConfiguration() throws MetadataException {
		VTAMetadataImportConfiguration conf = new VTAMetadataImportConfiguration(this);
		conf.setLocation(this.getLocation());
		this.setMetadataImportConfiguration(conf);
		return conf;

	}


	protected void setContent(String content) {
		this.content = content;
	}

	public String getContent() {
		return content;
	}

	public void setTargetNameSpace(String targetNameSpace) {
		this.targetNameSpace = targetNameSpace;
	}

	public String getTargetNameSpace() {
		return this.targetNameSpace;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public String getHeader() {
		return header;
	}

	public void setCharSet(String charSet) {
		this.charSet = charSet;
	}

	public String getCharSet() {
		return charSet;
	}

	public BOType getBOType() {
		return boType;
	}

	public void setBOType(BOType boType) {
		this.boType = boType;
	}

	public String getAliasTag() {
		return aliasTag;
	}

	public void setAliasTag(String aliasTag) {
		this.aliasTag = aliasTag;
	}

	public String getXsdName() {
		return xsdName;
	}

	public void setXsdName(String xsdName) {
		this.xsdName = xsdName;
	}
	public void setContainedChild (boolean containedChild) {
		this.containedChild = containedChild;
	}
	
	public boolean getContainedChild() {
		return this.containedChild;
	}
	
	public boolean isPartOfSameXSD() {
		return partOfSameXSD;
	}

	public void setPartOfSameXSD(boolean partOfSameXSD) {
		this.partOfSameXSD = partOfSameXSD;
	}

}
