/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.discovery;

import java.util.logging.Level;

import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIMultiValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAMetadataSelection extends WBIMetadataSelectionImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private String artifactType = null;
	private String serviceType = null;
//	private String[] operationsWPS = {WBIInteractionSpec.CREATE_OP,WBIInteractionSpec.UPDATE_OP,WBIInteractionSpec.DELETE_OP,
//			WBIInteractionSpec.RETRIEVE_OP,WBIInteractionSpec.RETRIEVE_ALL_OP,WBIInteractionSpec.EXISTS_OP,WBIInteractionSpec.APPLY_CHANGES_OP};
//	private String[] operationsNonWPS = {WBIInteractionSpec.CREATE_OP,WBIInteractionSpec.UPDATE_OP,WBIInteractionSpec.DELETE_OP,
//			WBIInteractionSpec.RETRIEVE_OP,WBIInteractionSpec.RETRIEVE_ALL_OP,WBIInteractionSpec.EXISTS_OP};
	private String[] operationsWPS = {WBIInteractionSpec.CREATE_OP,WBIInteractionSpec.UPDATE_OP,WBIInteractionSpec.DELETE_OP,
			WBIInteractionSpec.RETRIEVE_OP,WBIInteractionSpec.APPLY_CHANGES_OP};
	private String[] operationsNonWPS = {WBIInteractionSpec.CREATE_OP,WBIInteractionSpec.UPDATE_OP,WBIInteractionSpec.DELETE_OP,
			WBIInteractionSpec.RETRIEVE_OP};
	
	public VTAMetadataSelection() {
		try {
			applySelectionProperties(null);
		} catch (Exception e) {
		}
	}

	public PropertyGroup createSelectionProperties() {
		VTAMetadataDiscovery.getLogUtils().traceMethodEntrance("VTAMetadataSelection", "createSelectionProperties");
		WBIPropertyGroupImpl pg = null;
		try {
			pg = new WBIPropertyGroupImpl("Selection Properties");
			pg.setDisplayName("SelectionProperties"); 
			pg.setDescription("SelectionProperties"); 
			
			MetadataConfigurationType[] types = WBIMetadataDiscoveryImpl.getMetadataConfiguration();

			for(int i=0;i<types.length;i++){
				if (types[i].equals(MetadataConfigurationType.GENERIC_RECORDS) || types[i].equals(MetadataConfigurationType.GENERATED_RECORDS)
						|| types[i].equals(MetadataConfigurationType.GENERATED_DATA_BINDING) || types[i].equals(MetadataConfigurationType.GENERIC_DATA_BINDING)) {
					artifactType = types[i].toString();
				}
				if(types[i].equals(MetadataConfigurationType.INBOUND_SERVICE) || types[i].equals(MetadataConfigurationType.OUTBOUND_SERVICE)){
					serviceType = types[i].toString();
				}
			}

			WBIMultiValuedPropertyImpl operationProp = new WBIMultiValuedPropertyImpl("Operations", String.class); //$NON-NLS-1$
			operationProp.setDisplayName("Operations"); 
			operationProp.setDescription("Operations");  
			operationProp.setRequired(true);
			String[] operations=null;
			if(serviceType.equals(MetadataConfigurationType.INBOUND_SERVICE.toString())){
				operations = new String[3];
				operations[0] ="Create";
				operations[1] ="Update";
				operations[2] ="Delete"; 
			}else{
				if(artifactType.equals(MetadataConfigurationType.GENERATED_DATA_BINDING.toString())){// is WPS
					operations = operationsWPS;
				}else{
					operations = operationsNonWPS;
				}			
			}
			operationProp.setValidValues(operations);
			for(int idx=0;idx<operations.length;idx++){
				operationProp.addValue(operations[idx]);
			}
			pg.addProperty(operationProp);

			//commit it out.
//			WBISingleValuedPropertyImpl nameSpaceProp = new WBISingleValuedPropertyImpl("NameSpace", String.class);
//			nameSpaceProp.setValue("http://www.ibm.com/xmlns/prod/websphere/j2ca/vta"); 
//			pg.addProperty(nameSpaceProp);
//			nameSpaceProp.setDisplayName("NameSpace"); 
//			nameSpaceProp.setDescription("NameSpace"); 
//			nameSpaceProp.setRequired(true);

			//Copy the applied properties to the new instance
			if (getAppliedSelectionProperties() != null)
				EMDUtil.copyValues(getAppliedSelectionProperties(), pg);
		
		} catch (MetadataException e) {
			WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.SEVERE, "VTAMetadataSelection", "createSelectionProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage());
		}
		VTAMetadataDiscovery.getLogUtils().traceMethodExit("VTAMetadataSelection", "createSelectionProperties");
		return pg;
	}

	public String getSelectedArtifactType(){
		return this.artifactType;
	}
	public String getSelectedServiceType(){
		return this.serviceType;
	}
}
