/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.discovery;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.logging.Level;

import com.ibm.j2ca.base.internal.bidi.WBIBiDiConstants;
import com.ibm.j2ca.base.xsdutil.BOSchema;
import com.ibm.j2ca.base.xsdutil.BOType;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataObjectResponseImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataTreeImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.extensions.WBIFolderProperty;
import com.ibm.j2ca.extension.emd.dtf.discovery.DTFUtils;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.MetadataObject;
import commonj.connector.metadata.discovery.MetadataObjectResponse;
import commonj.connector.metadata.discovery.MetadataSelection;
import commonj.connector.metadata.discovery.connection.MetadataConnection;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAMetadataTree extends WBIMetadataTreeImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private static Hashtable treeNodes = new Hashtable();
	private String xsdFolderPath = "";
	private PropertyNameHelper helper = null;

	public VTAMetadataTree(MetadataConnection connection,PropertyNameHelper helper) {
		super(connection,helper);
		this.helper = helper;
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataTree", "VTAMetadataTree");
		
		WBIOutboundConnectionConfigurationImpl conf = (WBIOutboundConnectionConfigurationImpl) connection.getConnectionCofiguration();
		PropertyGroup pg = conf.getAppliedProperties();
		WBIFolderProperty xsdFilesPathProp = (WBIFolderProperty) pg.getProperty("XSDFolder");
		xsdFolderPath = xsdFilesPathProp.getValue().toString();
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().trace(Level.ALL, "VTAMetadataTree", "VTAMetadataTree", xsdFolderPath);
		
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataTree", "VTAMetadataTree");
	}

	public MetadataSelection createMetaDataSelection() {
		return new VTAMetadataSelection();
	}

	public MetadataObject getMetadataObject(String locationId) {
		return (MetadataObject) treeNodes.get(locationId);
	}

	public MetadataObjectResponse listMetadataObjects(PropertyGroup filterParameters) throws MetadataException {
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataTree", "listMetadataObjects");

		WBIMetadataObjectResponseImpl response = new WBIMetadataObjectResponseImpl();
		ArrayList objects = new ArrayList();

		String[] fileNames = getFileNames(this.xsdFolderPath);
		String directory = new File(this.xsdFolderPath).getAbsolutePath();
		if (!directory.endsWith(File.separator)) {
			directory = directory + File.separator;
		}
		String fileLocation = null;
		ArrayList objectsRegular = null;
		if (fileNames != null && fileNames.length > 0) {
			for (int i = 0; i < fileNames.length; i++) {
				try {
					fileLocation = directory + fileNames[i];
					BOSchema schema = new BOSchema(fileLocation, WBIBiDiConstants.EMPTY_STR);
					Hashtable types = schema.getBOTypes();
					File file = new File(fileLocation);
					FileInputStream fs = new FileInputStream(file);
					int size = fs.available();
					byte[] bytes = new byte[size];
					fs.read(bytes);
					objectsRegular = prepareSelectionObjectList(bytes, types, fileLocation);
					objects.addAll(objectsRegular);
				} catch (Exception e) {
					if (helper.getLogUtils().isTraceEnabled(Level.FINE))
						helper.getLogUtils().trace(Level.SEVERE, "VTAMetadataTree", "listMetadataObjects", "Error in creating top level objects", e);
					throw new MetadataException("Unable to load objects " + e.getMessage(), e);
				}				
			}
		}
		response.setObjects(objects);
		
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataTree", "listMetadataObjects");
		
		return response;
	}

	private String[] getFileNames(String path) {
		String fileNameList[] = null;
		File currDir = new File(path);
		if (currDir.isDirectory() && currDir.canRead()) {
			fileNameList = currDir.list(createFileNameFilter());
		}
		return fileNameList;
	}

	private FilenameFilter createFileNameFilter() {
		return new FilenameFilter() {
			public boolean accept(File dir, String filename) {
				boolean retValue = false;
				if (filename.toLowerCase().endsWith(".xsd"))
					retValue = true;
				return retValue;
			}
		};
	}

	private ArrayList prepareSelectionObjectList(byte[] bytes, Hashtable types, String fileLocation) throws MetadataException{
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataTree", "prepareSelectionObjectList");
		
		ArrayList objects = new ArrayList();
		String dataContent = new String(bytes);
		try {
			int encoding = dataContent.indexOf("encoding");
			if (encoding != -1) {
				String encodingName = DTFUtils.getSubstring(dataContent, encoding + 10, dataContent.indexOf("\"", encoding + 10));
				if (encodingName.length() > 0) {
					dataContent = new String(bytes, encodingName);
				}
			}
			// Find the targetNamespace
			int targetNSIndex = dataContent.indexOf("targetNamespace=");
			String targetNameSpace = DTFUtils.getSubstring(dataContent, targetNSIndex + 17, dataContent.indexOf("\"", targetNSIndex + 18));

			// preprocess to remove comments
			while (true) {
				int index = dataContent.indexOf("<!--");
				if (index != -1) {
					int endingIndex = dataContent.indexOf("-->", index);
					dataContent = DTFUtils.getSubstring(dataContent, 0, index) + dataContent.substring(endingIndex + 4);
				} else
					break;
			} 
			// Check if the file contains complexTypes
			int compIndex = dataContent.indexOf("complexType");
			if (compIndex == -1)
				throw new MetadataException("Invalid XSD document, missing header information");

			int index = DTFUtils.getSubstring(dataContent, 0, compIndex).lastIndexOf("<");

			String header = DTFUtils.getSubstring(dataContent, 0, index - 1);
			String matchTag = DTFUtils.getSubstring(dataContent, index + 1, compIndex);

			int footerIndex = dataContent.lastIndexOf("</" + matchTag + "complexType>");
			// Store the footer to be added to each individual complexType defn
			String footer = dataContent.substring(footerIndex + 16 + matchTag.length());

			// Extract the contents of the document starting from first complexType to last complexType
			String document = DTFUtils.getSubstring(dataContent, index, footerIndex + 16 + matchTag.length());
			while (true) {
				// parse the document backwards for "complexType"
				index = document.lastIndexOf("<" + matchTag + "complexType");
				if (index < 0)
					break;

				String token = document.substring(index);

				int endingIndex = token.indexOf(">");
				String complexTypeString = DTFUtils.getSubstring(token, 0, endingIndex);
				int nameIndex = complexTypeString.indexOf("name=");
				if (nameIndex == -1) {
					document = DTFUtils.getSubstring(document, 0, index);
					continue;
				}

				String name = DTFUtils.getSubstring(complexTypeString, nameIndex + 6, token.indexOf("\"", nameIndex + 7));
				VTAMetadataObject object = new VTAMetadataObject(this,helper);

				object.setDisplayName(name);
				object.setHeader(header);
				object.setAliasTag(matchTag);
				//object.setContentType(contentType);
				//object.setCharSet(charSet);
				object.setBOName(name);
				object.setLocation(targetNameSpace + name);
				object.setXsdName(fileLocation);

				treeNodes.put(object.getLocation(), object);
				
				VTAMetadataImportConfiguration conf = new VTAMetadataImportConfiguration(object);
				conf.setLocation(object.getLocation());
				object.setMetadataImportConfiguration(conf);
				object.setContent(System.getProperty("line.separator") + token + footer);
				object.setTargetNameSpace(targetNameSpace);
				// Get the BOType to associate
				// This is needed to link the parent child relationships
				BOType type = (BOType) types.get(name);
				if(type != null){
					object.setBOType(type);					
					if (type.getChildren() != null && type.getChildren().size() > 0) {
						object.setHasChildren(true);
					}
					object.setSelectableForImport(true);
					objects.add(object);
					// Continue till we find complexType defns
					if (index > 1)
						document = DTFUtils.getSubstring(document, 0, index);
					else
						break;
				}else{
					document = DTFUtils.getSubstring(document, 0, index);
					continue;
				}
			}
		} catch (Exception e) {
			throw new MetadataException(e.getMessage());
		}
		
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataTree", "prepareSelectionObjectList");
		return objects;

	}
	public PropertyGroup createFilterProperties() {
		return null;
	}

}
