/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.discovery.connection;

import java.util.logging.Level;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.vta.VTAActivationSpecWithXid;
import com.ibm.j2ca.vta.VTAResourceAdapter;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyDescriptor;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAInboundConnectionConfiguration extends WBIInboundConnectionConfigurationImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	PropertyNameHelper helper = null;
	
	public VTAInboundConnectionConfiguration(WBIInboundConnectionTypeImpl connType, PropertyNameHelper helper) throws MetadataException {
		super(connType);
		this.helper = helper;
	}

	public PropertyGroup createUnifiedProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionConfiguration", "createUnifiedProperties");

		WBIPropertyGroupImpl propGroup = null;
		try {
			propGroup = (WBIPropertyGroupImpl) createActivationSpecProperties();
			WBIPropertyGroupImpl adapterProp = (WBIPropertyGroupImpl) createResourceAdapterProperties();
			WBISingleValuedPropertyImpl logProp = (WBISingleValuedPropertyImpl) adapterProp.getProperties()[0];
			if (logProp != null) {
				if (logProp != null)
					logProp.setValue("VisualTestAdapter");
			}
			if (adapterProp != null) {
				PropertyDescriptor[] properties = adapterProp.getProperties();
				for (int i=0; i< adapterProp.getProperties().length; i++)
					propGroup.addProperty(properties[i]);
			}
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAInboundConnectionConfiguration", "createUnifiedProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionConfiguration", "createUnifiedProperties");

		return propGroup;
	}

	public PropertyGroup createActivationSpecProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionConfiguration", "createActivationSpecProperties");

		WBIPropertyGroupImpl connProp = null;
		try {
			
			EMDUtil util = new EMDUtil();
			connProp = (WBIPropertyGroupImpl) util.getPropertyGroup(new VTAActivationSpecWithXid(), helper);
			WBISingleValuedPropertyImpl namespaceProp = (WBISingleValuedPropertyImpl) connProp.getProperty("BONamespace");
			if (namespaceProp != null)
				connProp.remove(namespaceProp);
			
			WBIPropertyGroupImpl machineCredPG = new WBIPropertyGroupImpl("MachineCredentials", helper);
			machineCredPG.setDisplayName(helper.getPropertyName("MachineCredentials"));
			machineCredPG.setDescription(helper.getPropertyDescription("MachineCredentials")); 

			WBISingleValuedPropertyImpl prop = new WBISingleValuedPropertyImpl("Hostname", String.class);  
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("HostName"));  
			prop.setDescription(helper.getPropertyDescription("HostName"));
			prop.setValue("localhost");
			machineCredPG.addProperty(prop);

			prop = new WBISingleValuedPropertyImpl("Portnumber", String.class);  
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("Port"));  
			prop.setDescription(helper.getPropertyDescription("Port"));
			prop.setValue("9000");
			machineCredPG.addProperty(prop);
			
			connProp.addProperty(machineCredPG);

			if (getAppliedProperties() != null)
				util.copyValues(getAppliedProperties(), connProp,helper);
			
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAInboundConnectionConfiguration", "createActivationSpecProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}

		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionConfiguration", "createActivationSpecProperties");

		return connProp;
	}

	public PropertyGroup createResourceAdapterProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionConfiguration", "createResourceAdapterProperties");

		WBIPropertyGroupImpl propGroup = null;
		try {
			VTAResourceAdapter ra = new VTAResourceAdapter();
			EMDUtil util = new EMDUtil();
			propGroup = (WBIPropertyGroupImpl) util.getPropertyGroup(ra, helper);

			propGroup.setExpert(true);
			if (getAppliedProperties() != null && propGroup != null)
				EMDUtil.copyValues(getAppliedProperties(), propGroup);
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAInboundConnectionConfiguration", "createResourceAdapterProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}

		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionConfiguration", "createResourceAdapterProperties");
		return propGroup;
	}
}
