/*
 * IBM Confidential
 * OCO Source Materials
 * --------------
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.vta.emd.discovery.connection;

import java.util.logging.Level;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.extensions.WBIFolderProperty;
import com.ibm.j2ca.vta.VTAAdapterConstants;
import com.ibm.j2ca.vta.VTAResourceAdapter;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

public class VTAOutboundConnectionConfiguration extends WBIOutboundConnectionConfigurationImpl {
	static String copyright() {
		 return com.ibm.j2ca.vta.common.Copyright.IBM_COPYRIGHT_SHORT;
	}

	private PropertyNameHelper helper = null;
	
	public VTAOutboundConnectionConfiguration(WBIOutboundConnectionTypeImpl connType,PropertyNameHelper helper) throws MetadataException {
		super(connType, helper);
		this.helper = helper;	
	}

	public PropertyGroup createUnifiedProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAOutboundConnectionConfiguration", "createUnifiedProperties");
		
		WBIPropertyGroupImpl propGroup = null;
		try {
			if (this.getOutboundConnectionType().isSupportedInMetadataService()) {

				propGroup = new WBIPropertyGroupImpl("Connection Properties");
				propGroup.setDisplayName(helper.getPropertyName("ConnectionProperties"));
				propGroup.setDescription(helper.getPropertyDescription("ConnectionProperties"));
				
				WBIFolderProperty propFolder = new WBIFolderProperty("XSDFolder", java.io.File.class);
				propFolder.setRequired(true);
				propFolder.setMustExist(true);
				propFolder.setDisplayName(helper.getPropertyName("XSDFolder"));
				//CR91856
				propFolder.setDescription(helper.getPropertyDescription("XSDFolder"));
				
				propGroup.addProperty(propFolder);
				
			} else {
				propGroup = (WBIPropertyGroupImpl) createManagedConnectionFactoryProperties();
				
				PropertyGroup adapterProp = createResourceAdapterProperties();

				if(adapterProp != null){
					WBISingleValuedPropertyImpl logProp = (WBISingleValuedPropertyImpl) adapterProp.getProperties()[0];
					if (logProp != null)
					{
						logProp.setValue(VTAAdapterConstants.VTA_ADAPTERID);
					}
					propGroup.addProperty(adapterProp);
				}

			}
			if (this.getOutboundConnectionType().isSupportedInMetadataService() && (getAppliedProperties() != null)) {
				EMDUtil.copyValues(getAppliedProperties(), propGroup);
			}

		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAOutboundConnectionConfiguration", "createUnifiedProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAOutboundConnectionConfiguration", "createUnifiedProperties");
		return propGroup;
	}

	public PropertyGroup createManagedConnectionFactoryProperties() {
		//CR91877 - add checking for helper.getLogUtils()!=null
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAOutboundConnectionConfiguration", "createManagedConnectionFactoryProperties");

		WBIPropertyGroupImpl propGroup = null;
		try {
			propGroup = new WBIPropertyGroupImpl("Connection Properties");
			propGroup.setDisplayName(helper.getPropertyName("ConnectionProperties"));
			propGroup.setDescription(helper.getPropertyDescription("ConnectionProperties"));

			WBIPropertyGroupImpl mcPG = new WBIPropertyGroupImpl("MachineCredentials", helper); 
			mcPG.setDisplayName(helper.getPropertyName("MachineCredentials"));
			mcPG.setDescription(helper.getPropertyDescription("MachineCredentials")); 

			WBISingleValuedPropertyImpl prop = new WBISingleValuedPropertyImpl("Hostname", String.class);
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("HostName"));  
			prop.setDescription(helper.getPropertyDescription("HostName"));
			prop.setValue("localhost");
			mcPG.addProperty(prop);
			prop = new WBISingleValuedPropertyImpl("Portnumber", String.class);
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("Port"));  
			prop.setDescription(helper.getPropertyDescription("Port"));
			prop.setValue("9000");
			mcPG.addProperty(prop);
			
			propGroup.addProperty(mcPG);

			
			if (getAppliedProperties() != null)
				EMDUtil.copyValues(getAppliedProperties(), propGroup);

		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAOutboundConnectionConfiguration", "createManagedConnectionFactoryProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAOutboundConnectionConfiguration", "createManagedConnectionFactoryProperties");
		return propGroup;
	}

	public PropertyGroup createResourceAdapterProperties() {
		WBIPropertyGroupImpl propGroup = null;
		try {
			VTAResourceAdapter ra = new VTAResourceAdapter();
			EMDUtil util = new EMDUtil();
			propGroup = (WBIPropertyGroupImpl) util.getPropertyGroup(ra, helper);

			propGroup.setExpert(true);
			if (getAppliedProperties() != null && propGroup != null)
				EMDUtil.copyValues(getAppliedProperties(), propGroup);
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAOutboundConnectionConfiguration", "createResourceAdapterProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}

		
		return propGroup;
	}

}
